@@ -2,38 +2,21 @@ use std::any::type_name;
 use std::borrow::Cow;
 use std::fmt;
 
-use ahash::AHashMap;
-use num_bigint::BigInt;
-use pyo3::exceptions::{PyKeyError, PyTypeError, PyValueError};
+use pyo3::exceptions::{PyKeyError, PyTypeError};
 use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList};
 
-use crate::input::Int;
-use crate::tools::{extract_i64, py_err, py_error_type};
+use ahash::AHashMap;
+use num_bigint::BigInt;
 use strum::{Display, EnumMessage, IntoEnumIterator};
 use strum_macros::EnumIter;
 
+use crate::input::{InputType, Int};
+use crate::tools::{extract_i64, py_err, py_error_type};
+
 use super::PydanticCustomError;
 
-#[derive(Clone, Debug)]
-pub enum ErrorMode {
-    Python,
-    Json,
-}
-
-impl TryFrom<&str> for ErrorMode {
-    type Error = PyErr;
-
-    fn try_from(error_mode: &str) -> PyResult<Self> {
-        match error_mode {
-            "python" => Ok(Self::Python),
-            "json" => Ok(Self::Json),
-            s => py_err!(PyValueError; "Invalid error mode: {}", s),
-        }
-    }
-}
-
 #[pyfunction]
 pub fn list_all_errors(py: Python) -> PyResult<&PyList> {
     let mut errors: Vec<&PyDict> = Vec::with_capacity(100);
@@ -45,12 +28,12 @@ pub fn list_all_errors(py: Python) -> PyResult<&PyList> {
             d.set_item("message_template_python", message_template_python)?;
             d.set_item(
                 "example_message_python",
-                error_type.render_message(py, &ErrorMode::Python)?,
+                error_type.render_message(py, InputType::Python)?,
             )?;
             let message_template_json = error_type.message_template_json();
             if message_template_python != message_template_json {
                 d.set_item("message_template_json", message_template_json)?;
-                d.set_item("example_message_json", error_type.render_message(py, &ErrorMode::Json)?)?;
+                d.set_item("example_message_json", error_type.render_message(py, InputType::Json)?)?;
             }
             d.set_item("example_context", error_type.py_dict(py)?)?;
             errors.push(d);
@@ -67,7 +50,7 @@ fn field_from_context<'py, T: FromPyObject<'py>>(
 ) -> PyResult<T> {
     context
         .ok_or_else(|| py_error_type!(PyTypeError; "{}: '{}' required in context", enum_name, field_name))?
-        .get_item(field_name)
+        .get_item(field_name)?
         .ok_or_else(|| py_error_type!(PyTypeError; "{}: '{}' required in context", enum_name, field_name))?
         .extract::<T>()
         .map_err(|_| py_error_type!(PyTypeError; "{}: '{}' context value must be a {}", enum_name, field_name, type_name_fn()))
@@ -244,7 +227,7 @@ error_types! {
     TooLong {
         field_type: {ctx_type: String, ctx_fn: field_from_context},
         max_length: {ctx_type: usize, ctx_fn: field_from_context},
-        actual_length: {ctx_type: usize, ctx_fn: field_from_context},
+        actual_length: {ctx_type: Option<usize>, ctx_fn: field_from_context},
     },
     // ---------------------
     // generic collection and iteration errors
@@ -462,8 +445,8 @@ macro_rules! to_string_render {
     };
 }
 
-fn plural_s(value: usize) -> &'static str {
-    if value == 1 {
+fn plural_s<T: From<u8> + PartialEq>(value: T) -> &'static str {
+    if value == 1.into() {
         ""
     } else {
         "s"
@@ -511,8 +494,8 @@ impl ErrorType {
             Self::StringType {..} => "Input should be a valid string",
             Self::StringSubType {..} => "Input should be a string, not an instance of a subclass of str",
             Self::StringUnicode {..} => "Input should be a valid string, unable to parse raw data as a unicode string",
-            Self::StringTooShort {..} => "String should have at least {min_length} characters",
-            Self::StringTooLong {..} => "String should have at most {max_length} characters",
+            Self::StringTooShort {..} => "String should have at least {min_length} character{expected_plural}",
+            Self::StringTooLong {..} => "String should have at most {max_length} character{expected_plural}",
             Self::StringPatternMismatch {..} => "String should match pattern '{pattern}'",
             Self::Enum {..} => "Input should be {expected}",
             Self::DictType {..} => "Input should be a valid dictionary",
@@ -529,8 +512,8 @@ impl ErrorType {
             Self::FloatType {..} => "Input should be a valid number",
             Self::FloatParsing {..} => "Input should be a valid number, unable to parse string as a number",
             Self::BytesType {..} => "Input should be a valid bytes",
-            Self::BytesTooShort {..} => "Data should have at least {min_length} bytes",
-            Self::BytesTooLong {..} => "Data should have at most {max_length} bytes",
+            Self::BytesTooShort {..} => "Data should have at least {min_length} byte{expected_plural}",
+            Self::BytesTooLong {..} => "Data should have at most {max_length} byte{expected_plural}",
             Self::ValueError {..} => "Value error, {error}",
             Self::AssertionError {..} => "Assertion failed, {error}",
             Self::CustomError {..} => "",  // custom errors are handled separately
@@ -569,16 +552,16 @@ impl ErrorType {
             Self::UrlType {..} => "URL input should be a string or URL",
             Self::UrlParsing {..} => "Input should be a valid URL, {error}",
             Self::UrlSyntaxViolation {..} => "Input violated strict URL syntax rules, {error}",
-            Self::UrlTooLong {..} => "URL should have at most {max_length} characters",
+            Self::UrlTooLong {..} => "URL should have at most {max_length} character{expected_plural}",
             Self::UrlScheme {..} => "URL scheme should be {expected_schemes}",
             Self::UuidType {..} => "UUID input should be a string, bytes or UUID object",
             Self::UuidParsing {..} => "Input should be a valid UUID, {error}",
             Self::UuidVersion {..} => "UUID version {expected_version} expected",
             Self::DecimalType {..} => "Decimal input should be an integer, float, string or Decimal object",
             Self::DecimalParsing {..} => "Input should be a valid decimal",
-            Self::DecimalMaxDigits {..} => "Decimal input should have no more than {max_digits} digits in total",
-            Self::DecimalMaxPlaces {..} => "Decimal input should have no more than {decimal_places} decimal places",
-            Self::DecimalWholeDigits {..} => "Decimal input should have no more than {whole_digits} digits before the decimal point",
+            Self::DecimalMaxDigits {..} => "Decimal input should have no more than {max_digits} digit{expected_plural} in total",
+            Self::DecimalMaxPlaces {..} => "Decimal input should have no more than {decimal_places} decimal place{expected_plural}",
+            Self::DecimalWholeDigits {..} => "Decimal input should have no more than {whole_digits} digit{expected_plural} before the decimal point",
         }
     }
 
@@ -623,10 +606,10 @@ impl ErrorType {
         }
     }
 
-    pub fn render_message(&self, py: Python, error_mode: &ErrorMode) -> PyResult<String> {
-        let tmpl = match error_mode {
-            ErrorMode::Python => self.message_template_python(),
-            ErrorMode::Json => self.message_template_json(),
+    pub fn render_message(&self, py: Python, input_type: InputType) -> PyResult<String> {
+        let tmpl = match input_type {
+            InputType::Python => self.message_template_python(),
+            _ => self.message_template_json(),
         };
         match self {
             Self::NoSuchAttribute { attribute, .. } => render!(tmpl, attribute),
@@ -647,7 +630,7 @@ impl ErrorType {
                 ..
             } => {
                 let expected_plural = plural_s(*min_length);
-                to_string_render!(tmpl, field_type, min_length, actual_length, expected_plural)
+                to_string_render!(tmpl, field_type, min_length, actual_length, expected_plural,)
             }
             Self::TooLong {
                 field_type,
@@ -656,16 +639,29 @@ impl ErrorType {
                 ..
             } => {
                 let expected_plural = plural_s(*max_length);
-                to_string_render!(tmpl, field_type, max_length, actual_length, expected_plural)
+                let actual_length = actual_length.map_or(Cow::Borrowed("more"), |v| Cow::Owned(v.to_string()));
+                to_string_render!(tmpl, field_type, max_length, actual_length, expected_plural,)
             }
             Self::IterationError { error, .. } => render!(tmpl, error),
-            Self::StringTooShort { min_length, .. } => to_string_render!(tmpl, min_length),
-            Self::StringTooLong { max_length, .. } => to_string_render!(tmpl, max_length),
+            Self::StringTooShort { min_length, .. } => {
+                let expected_plural = plural_s(*min_length);
+                to_string_render!(tmpl, min_length, expected_plural)
+            }
+            Self::StringTooLong { max_length, .. } => {
+                let expected_plural = plural_s(*max_length);
+                to_string_render!(tmpl, max_length, expected_plural)
+            }
             Self::StringPatternMismatch { pattern, .. } => render!(tmpl, pattern),
             Self::Enum { expected, .. } => to_string_render!(tmpl, expected),
             Self::MappingType { error, .. } => render!(tmpl, error),
-            Self::BytesTooShort { min_length, .. } => to_string_render!(tmpl, min_length),
-            Self::BytesTooLong { max_length, .. } => to_string_render!(tmpl, max_length),
+            Self::BytesTooShort { min_length, .. } => {
+                let expected_plural = plural_s(*min_length);
+                to_string_render!(tmpl, min_length, expected_plural)
+            }
+            Self::BytesTooLong { max_length, .. } => {
+                let expected_plural = plural_s(*max_length);
+                to_string_render!(tmpl, max_length, expected_plural)
+            }
             Self::ValueError { error, .. } => {
                 let error = &error
                     .as_ref()
@@ -704,13 +700,25 @@ impl ErrorType {
             Self::UnionTagNotFound { discriminator, .. } => render!(tmpl, discriminator),
             Self::UrlParsing { error, .. } => render!(tmpl, error),
             Self::UrlSyntaxViolation { error, .. } => render!(tmpl, error),
-            Self::UrlTooLong { max_length, .. } => to_string_render!(tmpl, max_length),
+            Self::UrlTooLong { max_length, .. } => {
+                let expected_plural = plural_s(*max_length);
+                to_string_render!(tmpl, max_length, expected_plural)
+            }
             Self::UrlScheme { expected_schemes, .. } => render!(tmpl, expected_schemes),
             Self::UuidParsing { error, .. } => render!(tmpl, error),
             Self::UuidVersion { expected_version, .. } => to_string_render!(tmpl, expected_version),
-            Self::DecimalMaxDigits { max_digits, .. } => to_string_render!(tmpl, max_digits),
-            Self::DecimalMaxPlaces { decimal_places, .. } => to_string_render!(tmpl, decimal_places),
-            Self::DecimalWholeDigits { whole_digits, .. } => to_string_render!(tmpl, whole_digits),
+            Self::DecimalMaxDigits { max_digits, .. } => {
+                let expected_plural = plural_s(*max_digits);
+                to_string_render!(tmpl, max_digits, expected_plural)
+            }
+            Self::DecimalMaxPlaces { decimal_places, .. } => {
+                let expected_plural = plural_s(*decimal_places);
+                to_string_render!(tmpl, decimal_places, expected_plural)
+            }
+            Self::DecimalWholeDigits { whole_digits, .. } => {
+                let expected_plural = plural_s(*whole_digits);
+                to_string_render!(tmpl, whole_digits, expected_plural)
+            }
             _ => Ok(tmpl.to_string()),
         }
     }
