@@ -8,12 +8,15 @@ use pyo3::types::{
 };
 #[cfg(not(PyPy))]
 use pyo3::types::{PyDictItems, PyDictKeys, PyDictValues};
-use pyo3::{intern, AsPyPointer, PyTypeInfo};
+use pyo3::{intern, PyTypeInfo};
+
+use jiter::JsonValue;
 use speedate::MicrosecondsPrecisionOverflowBehavior;
 
-use crate::errors::{ErrorType, ErrorTypeDefaults, InputValue, LocItem, ValError, ValResult};
+use crate::errors::{AsLocItem, ErrorType, ErrorTypeDefaults, InputValue, LocItem, ValError, ValResult};
 use crate::tools::{extract_i64, safe_repr};
-use crate::validators::decimal::create_decimal;
+use crate::validators::decimal::{create_decimal, get_decimal_type};
+use crate::validators::Exactness;
 use crate::{ArgsKwargs, PyMultiHostUrl, PyUrl};
 
 use super::datetime::{
@@ -21,10 +24,14 @@ use super::datetime::{
     float_as_duration, float_as_time, int_as_datetime, int_as_duration, int_as_time, EitherDate, EitherDateTime,
     EitherTime,
 };
-use super::shared::{float_as_int, int_as_bool, map_json_err, str_as_bool, str_as_float, str_as_int};
+use super::return_enums::ValidationMatch;
+use super::shared::{
+    decimal_as_int, float_as_int, get_enum_meta_object, int_as_bool, map_json_err, str_as_bool, str_as_float,
+    str_as_int,
+};
 use super::{
-    py_string_str, EitherBytes, EitherFloat, EitherInt, EitherString, EitherTimedelta, GenericArguments,
-    GenericIterable, GenericIterator, GenericMapping, Input, JsonInput, PyArgs,
+    py_string_str, BorrowInput, EitherBytes, EitherFloat, EitherInt, EitherString, EitherTimedelta, GenericArguments,
+    GenericIterable, GenericIterator, GenericMapping, Input, PyArgs,
 };
 
 #[cfg(not(PyPy))]
@@ -32,7 +39,7 @@ macro_rules! extract_dict_keys {
     ($py:expr, $obj:ident) => {
         $obj.downcast::<PyDictKeys>()
             .ok()
-            .map(|v| PyIterator::from_object($py, v).unwrap())
+            .map(|v| PyIterator::from_object(v).unwrap())
     };
 }
 
@@ -40,7 +47,7 @@ macro_rules! extract_dict_keys {
 macro_rules! extract_dict_keys {
     ($py:expr, $obj:ident) => {
         if is_dict_keys_type($obj) {
-            Some(PyIterator::from_object($py, $obj).unwrap())
+            Some(PyIterator::from_object($obj).unwrap())
         } else {
             None
         }
@@ -52,7 +59,7 @@ macro_rules! extract_dict_values {
     ($py:expr, $obj:ident) => {
         $obj.downcast::<PyDictValues>()
             .ok()
-            .map(|v| PyIterator::from_object($py, v).unwrap())
+            .map(|v| PyIterator::from_object(v).unwrap())
     };
 }
 
@@ -60,7 +67,7 @@ macro_rules! extract_dict_values {
 macro_rules! extract_dict_values {
     ($py:expr, $obj:ident) => {
         if is_dict_values_type($obj) {
-            Some(PyIterator::from_object($py, $obj).unwrap())
+            Some(PyIterator::from_object($obj).unwrap())
         } else {
             None
         }
@@ -72,7 +79,7 @@ macro_rules! extract_dict_items {
     ($py:expr, $obj:ident) => {
         $obj.downcast::<PyDictItems>()
             .ok()
-            .map(|v| PyIterator::from_object($py, v).unwrap())
+            .map(|v| PyIterator::from_object(v).unwrap())
     };
 }
 
@@ -80,14 +87,14 @@ macro_rules! extract_dict_items {
 macro_rules! extract_dict_items {
     ($py:expr, $obj:ident) => {
         if is_dict_items_type($obj) {
-            Some(PyIterator::from_object($py, $obj).unwrap())
+            Some(PyIterator::from_object($obj).unwrap())
         } else {
             None
         }
     };
 }
 
-impl<'a> Input<'a> for PyAny {
+impl AsLocItem for PyAny {
     fn as_loc_item(&self) -> LocItem {
         if let Ok(py_str) = self.downcast::<PyString>() {
             py_str.to_string_lossy().as_ref().into()
@@ -97,7 +104,9 @@ impl<'a> Input<'a> for PyAny {
             safe_repr(self).to_string().into()
         }
     }
+}
 
+impl<'a> Input<'a> for PyAny {
     fn as_error_value(&'a self) -> InputValue<'a> {
         InputValue::PyAny(self)
     }
@@ -180,29 +189,71 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    fn parse_json(&'a self) -> ValResult<'a, JsonInput> {
-        if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            serde_json::from_slice(py_bytes.as_bytes()).map_err(|e| map_json_err(self, e))
+    fn parse_json(&'a self) -> ValResult<'a, JsonValue> {
+        let bytes = if let Ok(py_bytes) = self.downcast::<PyBytes>() {
+            py_bytes.as_bytes()
         } else if let Ok(py_str) = self.downcast::<PyString>() {
-            let str = py_str.to_str()?;
-            serde_json::from_str(str).map_err(|e| map_json_err(self, e))
+            let str = py_string_str(py_str)?;
+            str.as_bytes()
         } else if let Ok(py_byte_array) = self.downcast::<PyByteArray>() {
-            serde_json::from_slice(unsafe { py_byte_array.as_bytes() }).map_err(|e| map_json_err(self, e))
+            // Safety: from_slice does not run arbitrary Python code and the GIL is held so the
+            // bytes array will not be mutated while `JsonValue::parse` is reading it
+            unsafe { py_byte_array.as_bytes() }
         } else {
-            Err(ValError::new(ErrorTypeDefaults::JsonType, self))
-        }
+            return Err(ValError::new(ErrorTypeDefaults::JsonType, self));
+        };
+        JsonValue::parse(bytes, true).map_err(|e| map_json_err(self, e))
     }
 
-    fn strict_str(&'a self) -> ValResult<EitherString<'a>> {
-        if let Ok(py_str) = <PyString as PyTryFrom>::try_from_exact(self) {
-            Ok(py_str.into())
+    fn validate_str(
+        &'a self,
+        strict: bool,
+        coerce_numbers_to_str: bool,
+    ) -> ValResult<ValidationMatch<EitherString<'a>>> {
+        if let Ok(py_str) = self.downcast_exact::<PyString>() {
+            return Ok(ValidationMatch::exact(py_str.into()));
         } else if let Ok(py_str) = self.downcast::<PyString>() {
             // force to a rust string to make sure behavior is consistent whether or not we go via a
             // rust string in StrConstrainedValidator - e.g. to_lower
-            Ok(py_string_str(py_str)?.into())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::StringType, self))
+            return Ok(ValidationMatch::strict(py_string_str(py_str)?.into()));
         }
+
+        'lax: {
+            if !strict {
+                return if let Ok(bytes) = self.downcast::<PyBytes>() {
+                    match from_utf8(bytes.as_bytes()) {
+                        Ok(str) => Ok(str.into()),
+                        Err(_) => Err(ValError::new(ErrorTypeDefaults::StringUnicode, self)),
+                    }
+                } else if let Ok(py_byte_array) = self.downcast::<PyByteArray>() {
+                    // Safety: the gil is held while from_utf8 is running so py_byte_array is not mutated,
+                    // and we immediately copy the bytes into a new Python string
+                    match from_utf8(unsafe { py_byte_array.as_bytes() }) {
+                        // Why Python not Rust? to avoid an unnecessary allocation on the Rust side, the
+                        // final output needs to be Python anyway.
+                        Ok(s) => Ok(PyString::new(self.py(), s).into()),
+                        Err(_) => Err(ValError::new(ErrorTypeDefaults::StringUnicode, self)),
+                    }
+                } else if coerce_numbers_to_str && !PyBool::is_exact_type_of(self) && {
+                    let py = self.py();
+                    let decimal_type: Py<PyType> = get_decimal_type(py);
+
+                    // only allow int, float, and decimal (not bool)
+                    self.is_instance_of::<PyInt>()
+                        || self.is_instance_of::<PyFloat>()
+                        || self.is_instance(decimal_type.as_ref(py)).unwrap_or_default()
+                } {
+                    Ok(self.str()?.into())
+                } else if let Some(enum_val) = maybe_as_enum(self) {
+                    Ok(enum_val.str()?.into())
+                } else {
+                    break 'lax;
+                }
+                .map(ValidationMatch::lax);
+            }
+        }
+
+        Err(ValError::new(ErrorTypeDefaults::StringType, self))
     }
 
     fn exact_str(&'a self) -> ValResult<EitherString<'a>> {
@@ -221,146 +272,123 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    fn lax_str(&'a self) -> ValResult<EitherString<'a>> {
-        if let Ok(py_str) = <PyString as PyTryFrom>::try_from_exact(self) {
-            Ok(py_str.into())
-        } else if let Ok(py_str) = self.downcast::<PyString>() {
-            // force to a rust string to make sure behaviour is consistent whether or not we go via a
-            // rust string in StrConstrainedValidator - e.g. to_lower
-            Ok(py_string_str(py_str)?.into())
-        } else if let Ok(bytes) = self.downcast::<PyBytes>() {
-            let str = match from_utf8(bytes.as_bytes()) {
-                Ok(s) => s,
-                Err(_) => return Err(ValError::new(ErrorTypeDefaults::StringUnicode, self)),
-            };
-            Ok(str.into())
-        } else if let Ok(py_byte_array) = self.downcast::<PyByteArray>() {
-            // see https://docs.rs/pyo3/latest/pyo3/types/struct.PyByteArray.html#method.as_bytes
-            // for why this is marked unsafe
-            let str = match from_utf8(unsafe { py_byte_array.as_bytes() }) {
-                Ok(s) => s,
-                Err(_) => return Err(ValError::new(ErrorTypeDefaults::StringUnicode, self)),
-            };
-            Ok(str.into())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::StringType, self))
+    fn validate_bytes(&'a self, strict: bool) -> ValResult<ValidationMatch<EitherBytes<'a>>> {
+        if let Ok(py_bytes) = self.downcast_exact::<PyBytes>() {
+            return Ok(ValidationMatch::exact(py_bytes.into()));
+        } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
+            return Ok(ValidationMatch::strict(py_bytes.into()));
         }
-    }
 
-    fn strict_bytes(&'a self) -> ValResult<EitherBytes<'a>> {
-        if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            Ok(py_bytes.into())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::BytesType, self))
-        }
-    }
-
-    fn lax_bytes(&'a self) -> ValResult<EitherBytes<'a>> {
-        if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            Ok(py_bytes.into())
-        } else if let Ok(py_str) = self.downcast::<PyString>() {
-            let str = py_string_str(py_str)?;
-            Ok(str.as_bytes().into())
-        } else if let Ok(py_byte_array) = self.downcast::<PyByteArray>() {
-            Ok(py_byte_array.to_vec().into())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::BytesType, self))
-        }
-    }
-
-    fn strict_bool(&self) -> ValResult<bool> {
-        if let Ok(bool) = self.downcast::<PyBool>() {
-            Ok(bool.is_true())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::BoolType, self))
-        }
-    }
-
-    fn lax_bool(&self) -> ValResult<bool> {
-        if let Ok(bool) = self.downcast::<PyBool>() {
-            Ok(bool.is_true())
-        } else if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::BoolParsing)? {
-            str_as_bool(self, &cow_str)
-        } else if let Ok(int) = extract_i64(self) {
-            int_as_bool(self, int)
-        } else if let Ok(float) = self.extract::<f64>() {
-            match float_as_int(self, float) {
-                Ok(int) => int
-                    .as_bool()
-                    .ok_or_else(|| ValError::new(ErrorTypeDefaults::BoolParsing, self)),
-                _ => Err(ValError::new(ErrorTypeDefaults::BoolType, self)),
+        'lax: {
+            if !strict {
+                return if let Ok(py_str) = self.downcast::<PyString>() {
+                    let str = py_string_str(py_str)?;
+                    Ok(str.as_bytes().into())
+                } else if let Ok(py_byte_array) = self.downcast::<PyByteArray>() {
+                    Ok(py_byte_array.to_vec().into())
+                } else {
+                    break 'lax;
+                }
+                .map(ValidationMatch::lax);
             }
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::BoolType, self))
         }
+
+        Err(ValError::new(ErrorTypeDefaults::BytesType, self))
     }
 
-    fn strict_int(&'a self) -> ValResult<EitherInt<'a>> {
-        if PyInt::is_exact_type_of(self) {
-            Ok(EitherInt::Py(self))
-        } else if PyInt::is_type_of(self) {
+    fn validate_bool(&self, strict: bool) -> ValResult<'_, ValidationMatch<bool>> {
+        if let Ok(bool) = self.downcast::<PyBool>() {
+            return Ok(ValidationMatch::exact(bool.is_true()));
+        }
+
+        if !strict {
+            if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::BoolParsing)? {
+                return str_as_bool(self, &cow_str).map(ValidationMatch::lax);
+            } else if let Ok(int) = extract_i64(self) {
+                return int_as_bool(self, int).map(ValidationMatch::lax);
+            } else if let Ok(float) = self.extract::<f64>() {
+                if let Ok(int) = float_as_int(self, float) {
+                    return int
+                        .as_bool()
+                        .ok_or_else(|| ValError::new(ErrorTypeDefaults::BoolParsing, self))
+                        .map(ValidationMatch::lax);
+                };
+            }
+        }
+
+        Err(ValError::new(ErrorTypeDefaults::BoolType, self))
+    }
+
+    fn validate_int(&'a self, strict: bool) -> ValResult<'a, ValidationMatch<EitherInt<'a>>> {
+        if self.is_exact_instance_of::<PyInt>() {
+            return Ok(ValidationMatch::exact(EitherInt::Py(self)));
+        } else if self.is_instance_of::<PyInt>() {
             // bools are a subclass of int, so check for bool type in this specific case
-            if PyBool::is_exact_type_of(self) {
-                Err(ValError::new(ErrorTypeDefaults::IntType, self))
+            let exactness = if self.is_instance_of::<PyBool>() {
+                if strict {
+                    return Err(ValError::new(ErrorTypeDefaults::IntType, self));
+                }
+                Exactness::Lax
             } else {
-                Ok(EitherInt::Py(self))
+                Exactness::Strict
+            };
+
+            // force to an int to upcast to a pure python int
+            return EitherInt::upcast(self).map(|either_int| ValidationMatch::new(either_int, exactness));
+        }
+
+        'lax: {
+            if !strict {
+                return if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::IntParsing)? {
+                    str_as_int(self, &cow_str)
+                } else if self.is_exact_instance_of::<PyFloat>() {
+                    float_as_int(self, self.extract::<f64>()?)
+                } else if let Ok(decimal) = self.strict_decimal(self.py()) {
+                    decimal_as_int(self.py(), self, decimal)
+                } else if let Ok(float) = self.extract::<f64>() {
+                    float_as_int(self, float)
+                } else if let Some(enum_val) = maybe_as_enum(self) {
+                    Ok(EitherInt::Py(enum_val))
+                } else {
+                    break 'lax;
+                }
+                .map(ValidationMatch::lax);
             }
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::IntType, self))
         }
+
+        Err(ValError::new(ErrorTypeDefaults::IntType, self))
     }
 
-    fn lax_int(&'a self) -> ValResult<EitherInt<'a>> {
-        if PyInt::is_exact_type_of(self) {
-            Ok(EitherInt::Py(self))
-        } else if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::IntParsing)? {
-            str_as_int(self, &cow_str)
-        } else if let Ok(float) = self.extract::<f64>() {
-            float_as_int(self, float)
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::IntType, self))
+    fn validate_float(&'a self, strict: bool) -> ValResult<'a, ValidationMatch<EitherFloat<'a>>> {
+        if let Ok(float) = self.downcast_exact::<PyFloat>() {
+            return Ok(ValidationMatch::exact(EitherFloat::Py(float)));
         }
-    }
 
-    fn ultra_strict_float(&'a self) -> ValResult<EitherFloat<'a>> {
-        if self.is_instance_of::<PyInt>() {
-            Err(ValError::new(ErrorTypeDefaults::FloatType, self))
-        } else if let Ok(float) = self.downcast::<PyFloat>() {
-            Ok(EitherFloat::Py(float))
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::FloatType, self))
+        if !strict {
+            if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::FloatParsing)? {
+                // checking for bytes and string is fast, so do this before isinstance(float)
+                return str_as_float(self, &cow_str).map(ValidationMatch::lax);
+            }
         }
-    }
-    fn strict_float(&'a self) -> ValResult<EitherFloat<'a>> {
-        if PyFloat::is_exact_type_of(self) {
-            // Safety: self is PyFloat
-            Ok(EitherFloat::Py(unsafe { self.downcast_unchecked::<PyFloat>() }))
-        } else if let Ok(float) = self.extract::<f64>() {
-            // bools are cast to floats as either 0.0 or 1.0, so check for bool type in this specific case
-            if (float == 0.0 || float == 1.0) && PyBool::is_exact_type_of(self) {
-                Err(ValError::new(ErrorTypeDefaults::FloatType, self))
+
+        if let Ok(float) = self.extract::<f64>() {
+            let exactness = if self.is_instance_of::<PyBool>() {
+                if strict {
+                    return Err(ValError::new(ErrorTypeDefaults::FloatType, self));
+                }
+                Exactness::Lax
             } else {
-                Ok(EitherFloat::F64(float))
-            }
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::FloatType, self))
+                Exactness::Strict
+            };
+            return Ok(ValidationMatch::new(EitherFloat::F64(float), exactness));
         }
+
+        Err(ValError::new(ErrorTypeDefaults::FloatType, self))
     }
 
-    fn lax_float(&'a self) -> ValResult<EitherFloat<'a>> {
-        if PyFloat::is_exact_type_of(self) {
-            // Safety: self is PyFloat
-            Ok(EitherFloat::Py(unsafe { self.downcast_unchecked::<PyFloat>() }))
-        } else if let Some(cow_str) = maybe_as_string(self, ErrorTypeDefaults::FloatParsing)? {
-            str_as_float(self, &cow_str)
-        } else if let Ok(float) = self.extract::<f64>() {
-            Ok(EitherFloat::F64(float))
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::FloatType, self))
-        }
-    }
-
-    fn strict_decimal(&'a self, decimal_type: &'a PyType) -> ValResult<&'a PyAny> {
+    fn strict_decimal(&'a self, py: Python<'a>) -> ValResult<&'a PyAny> {
+        let decimal_type_obj: Py<PyType> = get_decimal_type(py);
+        let decimal_type = decimal_type_obj.as_ref(py);
         // Fast path for existing decimal objects
         if self.is_exact_instance(decimal_type) {
             return Ok(self);
@@ -368,7 +396,7 @@ impl<'a> Input<'a> for PyAny {
 
         // Try subclasses of decimals, they will be upcast to Decimal
         if self.is_instance(decimal_type)? {
-            return create_decimal(self, self, decimal_type);
+            return create_decimal(self, self, py);
         }
 
         Err(ValError::new(
@@ -380,7 +408,9 @@ impl<'a> Input<'a> for PyAny {
         ))
     }
 
-    fn lax_decimal(&'a self, decimal_type: &'a PyType) -> ValResult<&'a PyAny> {
+    fn lax_decimal(&'a self, py: Python<'a>) -> ValResult<&'a PyAny> {
+        let decimal_type_obj: Py<PyType> = get_decimal_type(py);
+        let decimal_type = decimal_type_obj.as_ref(py);
         // Fast path for existing decimal objects
         if self.is_exact_instance(decimal_type) {
             return Ok(self);
@@ -388,12 +418,12 @@ impl<'a> Input<'a> for PyAny {
 
         if self.is_instance_of::<PyString>() || (self.is_instance_of::<PyInt>() && !self.is_instance_of::<PyBool>()) {
             // checking isinstance for str / int / bool is fast compared to decimal / float
-            create_decimal(self, self, decimal_type)
+            create_decimal(self, self, py)
         } else if self.is_instance(decimal_type)? {
             // upcast subclasses to decimal
-            return create_decimal(self, self, decimal_type);
+            return create_decimal(self, self, py);
         } else if self.is_instance_of::<PyFloat>() {
-            create_decimal(self.str()?, self, decimal_type)
+            create_decimal(self.str()?, self, py)
         } else {
             Err(ValError::new(ErrorTypeDefaults::DecimalType, self))
         }
@@ -570,128 +600,143 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    fn strict_date(&self) -> ValResult<EitherDate> {
-        if PyDateTime::is_type_of(self) {
-            // have to check if it's a datetime first, otherwise the line below converts to a date
-            Err(ValError::new(ErrorTypeDefaults::DateType, self))
-        } else if let Ok(date) = self.downcast::<PyDate>() {
-            Ok(date.into())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::DateType, self))
-        }
-    }
-
-    fn lax_date(&self) -> ValResult<EitherDate> {
-        if PyDateTime::is_type_of(self) {
+    fn validate_date(&self, strict: bool) -> ValResult<ValidationMatch<EitherDate>> {
+        if let Ok(date) = self.downcast_exact::<PyDate>() {
+            Ok(ValidationMatch::exact(date.into()))
+        } else if PyDateTime::is_type_of(self) {
             // have to check if it's a datetime first, otherwise the line below converts to a date
             // even if we later try coercion from a datetime, we don't want to return a datetime now
             Err(ValError::new(ErrorTypeDefaults::DateType, self))
         } else if let Ok(date) = self.downcast::<PyDate>() {
-            Ok(date.into())
-        } else if let Ok(py_str) = self.downcast::<PyString>() {
-            let str = py_string_str(py_str)?;
-            bytes_as_date(self, str.as_bytes())
-        } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            bytes_as_date(self, py_bytes.as_bytes())
+            Ok(ValidationMatch::strict(date.into()))
+        } else if let Some(bytes) = {
+            if strict {
+                None
+            } else if let Ok(py_str) = self.downcast::<PyString>() {
+                let str = py_string_str(py_str)?;
+                Some(str.as_bytes())
+            } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
+                Some(py_bytes.as_bytes())
+            } else {
+                None
+            }
+        } {
+            bytes_as_date(self, bytes).map(ValidationMatch::lax)
         } else {
             Err(ValError::new(ErrorTypeDefaults::DateType, self))
         }
     }
 
-    fn strict_time(
-        &self,
-        _microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
-    ) -> ValResult<EitherTime> {
-        if let Ok(time) = self.downcast::<PyTime>() {
-            Ok(time.into())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::TimeType, self))
-        }
-    }
-
-    fn lax_time(&self, microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior) -> ValResult<EitherTime> {
-        if let Ok(time) = self.downcast::<PyTime>() {
-            Ok(time.into())
-        } else if let Ok(py_str) = self.downcast::<PyString>() {
-            let str = py_string_str(py_str)?;
-            bytes_as_time(self, str.as_bytes(), microseconds_overflow_behavior)
-        } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            bytes_as_time(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
-        } else if PyBool::is_exact_type_of(self) {
-            Err(ValError::new(ErrorTypeDefaults::TimeType, self))
-        } else if let Ok(int) = extract_i64(self) {
-            int_as_time(self, int, 0)
-        } else if let Ok(float) = self.extract::<f64>() {
-            float_as_time(self, float)
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::TimeType, self))
-        }
-    }
-
-    fn strict_datetime(
-        &self,
-        _microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
-    ) -> ValResult<EitherDateTime> {
-        if let Ok(dt) = self.downcast::<PyDateTime>() {
-            Ok(dt.into())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::DatetimeType, self))
-        }
-    }
-
-    fn lax_datetime(
+    fn validate_time(
         &self,
+        strict: bool,
         microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
-    ) -> ValResult<EitherDateTime> {
-        if let Ok(dt) = self.downcast::<PyDateTime>() {
-            Ok(dt.into())
-        } else if let Ok(py_str) = self.downcast::<PyString>() {
-            let str = py_string_str(py_str)?;
-            bytes_as_datetime(self, str.as_bytes(), microseconds_overflow_behavior)
-        } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            bytes_as_datetime(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
-        } else if PyBool::is_exact_type_of(self) {
-            Err(ValError::new(ErrorTypeDefaults::DatetimeType, self))
-        } else if let Ok(int) = extract_i64(self) {
-            int_as_datetime(self, int, 0)
-        } else if let Ok(float) = self.extract::<f64>() {
-            float_as_datetime(self, float)
-        } else if let Ok(date) = self.downcast::<PyDate>() {
-            Ok(date_as_datetime(date)?)
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::DatetimeType, self))
+    ) -> ValResult<ValidationMatch<EitherTime>> {
+        if let Ok(time) = self.downcast_exact::<PyTime>() {
+            return Ok(ValidationMatch::exact(time.into()));
+        } else if let Ok(time) = self.downcast::<PyTime>() {
+            return Ok(ValidationMatch::strict(time.into()));
         }
+
+        'lax: {
+            if !strict {
+                return if let Ok(py_str) = self.downcast::<PyString>() {
+                    let str = py_string_str(py_str)?;
+                    bytes_as_time(self, str.as_bytes(), microseconds_overflow_behavior)
+                } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
+                    bytes_as_time(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
+                } else if PyBool::is_exact_type_of(self) {
+                    Err(ValError::new(ErrorTypeDefaults::TimeType, self))
+                } else if let Ok(int) = extract_i64(self) {
+                    int_as_time(self, int, 0)
+                } else if let Ok(float) = self.extract::<f64>() {
+                    float_as_time(self, float)
+                } else {
+                    break 'lax;
+                }
+                .map(ValidationMatch::lax);
+            }
+        }
+
+        Err(ValError::new(ErrorTypeDefaults::TimeType, self))
     }
 
-    fn strict_timedelta(
-        &self,
-        _microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
-    ) -> ValResult<EitherTimedelta> {
-        if let Ok(either_dt) = EitherTimedelta::try_from(self) {
-            Ok(either_dt)
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::TimeDeltaType, self))
-        }
-    }
-
-    fn lax_timedelta(
+    fn validate_datetime(
         &self,
+        strict: bool,
         microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
-    ) -> ValResult<EitherTimedelta> {
-        if let Ok(either_dt) = EitherTimedelta::try_from(self) {
-            Ok(either_dt)
-        } else if let Ok(py_str) = self.downcast::<PyString>() {
-            let str = py_string_str(py_str)?;
-            bytes_as_timedelta(self, str.as_bytes(), microseconds_overflow_behavior)
-        } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
-            bytes_as_timedelta(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
-        } else if let Ok(int) = extract_i64(self) {
-            Ok(int_as_duration(self, int)?.into())
-        } else if let Ok(float) = self.extract::<f64>() {
-            Ok(float_as_duration(self, float)?.into())
-        } else {
-            Err(ValError::new(ErrorTypeDefaults::TimeDeltaType, self))
+    ) -> ValResult<ValidationMatch<EitherDateTime>> {
+        if let Ok(dt) = self.downcast_exact::<PyDateTime>() {
+            return Ok(ValidationMatch::exact(dt.into()));
+        } else if let Ok(dt) = self.downcast::<PyDateTime>() {
+            return Ok(ValidationMatch::strict(dt.into()));
         }
+
+        'lax: {
+            if !strict {
+                return if let Ok(py_str) = self.downcast::<PyString>() {
+                    let str = py_string_str(py_str)?;
+                    bytes_as_datetime(self, str.as_bytes(), microseconds_overflow_behavior)
+                } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
+                    bytes_as_datetime(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
+                } else if PyBool::is_exact_type_of(self) {
+                    Err(ValError::new(ErrorTypeDefaults::DatetimeType, self))
+                } else if let Ok(int) = extract_i64(self) {
+                    int_as_datetime(self, int, 0)
+                } else if let Ok(float) = self.extract::<f64>() {
+                    float_as_datetime(self, float)
+                } else if let Ok(date) = self.downcast::<PyDate>() {
+                    Ok(date_as_datetime(date)?)
+                } else {
+                    break 'lax;
+                }
+                .map(ValidationMatch::lax);
+            }
+        }
+
+        Err(ValError::new(ErrorTypeDefaults::DatetimeType, self))
+    }
+
+    fn validate_timedelta(
+        &self,
+        strict: bool,
+        microseconds_overflow_behavior: MicrosecondsPrecisionOverflowBehavior,
+    ) -> ValResult<ValidationMatch<EitherTimedelta>> {
+        if let Ok(either_dt) = EitherTimedelta::try_from(self) {
+            let exactness = if matches!(either_dt, EitherTimedelta::PyExact(_)) {
+                Exactness::Exact
+            } else {
+                Exactness::Strict
+            };
+            return Ok(ValidationMatch::new(either_dt, exactness));
+        }
+
+        'lax: {
+            if !strict {
+                return if let Ok(py_str) = self.downcast::<PyString>() {
+                    let str = py_string_str(py_str)?;
+                    bytes_as_timedelta(self, str.as_bytes(), microseconds_overflow_behavior)
+                } else if let Ok(py_bytes) = self.downcast::<PyBytes>() {
+                    bytes_as_timedelta(self, py_bytes.as_bytes(), microseconds_overflow_behavior)
+                } else if let Ok(int) = extract_i64(self) {
+                    Ok(int_as_duration(self, int)?.into())
+                } else if let Ok(float) = self.extract::<f64>() {
+                    Ok(float_as_duration(self, float)?.into())
+                } else {
+                    break 'lax;
+                }
+                .map(ValidationMatch::lax);
+            }
+        }
+
+        Err(ValError::new(ErrorTypeDefaults::TimeDeltaType, self))
+    }
+}
+
+impl BorrowInput for &'_ PyAny {
+    type Input<'a> = PyAny where Self: 'a;
+    fn borrow_input(&self) -> &Self::Input<'_> {
+        self
     }
 }
 
@@ -727,6 +772,18 @@ fn maybe_as_string(v: &PyAny, unicode_error: ErrorType) -> ValResult<Option<Cow<
     }
 }
 
+/// Utility for extracting an enum value, if possible.
+fn maybe_as_enum(v: &PyAny) -> Option<&PyAny> {
+    let py = v.py();
+    let enum_meta_object = get_enum_meta_object(py);
+    let meta_type = v.get_type().get_type();
+    if meta_type.is(&enum_meta_object) {
+        v.getattr(intern!(py, "value")).ok()
+    } else {
+        None
+    }
+}
+
 #[cfg(PyPy)]
 static DICT_KEYS_TYPE: pyo3::once_cell::GILOnceCell<Py<PyType>> = pyo3::once_cell::GILOnceCell::new();
 
