@@ -4,6 +4,7 @@ use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList};
 
+use crate::definitions::DefinitionRef;
 use crate::definitions::DefinitionsBuilder;
 
 use crate::tools::SchemaDict;
@@ -25,9 +26,12 @@ impl BuildSerializer for DefinitionsSerializerBuilder {
 
         let schema_definitions: &PyList = schema.get_as_req(intern!(py, "definitions"))?;
 
-        for schema_def in schema_definitions {
-            CombinedSerializer::build(schema_def.downcast()?, config, definitions)?;
-            // no need to store the serializer here, it has already been stored in definitions if necessary
+        for schema_definition in schema_definitions {
+            let reference = schema_definition
+                .extract::<&PyDict>()?
+                .get_as_req::<String>(intern!(py, "ref"))?;
+            let serializer = CombinedSerializer::build(schema_definition.downcast()?, config, definitions)?;
+            definitions.add_definition(reference, serializer)?;
         }
 
         let inner_schema: &PyDict = schema.get_as_req(intern!(py, "schema"))?;
@@ -37,13 +41,7 @@ impl BuildSerializer for DefinitionsSerializerBuilder {
 
 #[derive(Debug, Clone)]
 pub struct DefinitionRefSerializer {
-    serializer_id: usize,
-}
-
-impl DefinitionRefSerializer {
-    pub fn from_id(serializer_id: usize) -> CombinedSerializer {
-        Self { serializer_id }.into()
-    }
+    definition: DefinitionRef<CombinedSerializer>,
 }
 
 impl BuildSerializer for DefinitionRefSerializer {
@@ -54,9 +52,9 @@ impl BuildSerializer for DefinitionRefSerializer {
         _config: Option<&PyDict>,
         definitions: &mut DefinitionsBuilder<CombinedSerializer>,
     ) -> PyResult<CombinedSerializer> {
-        let schema_ref: String = schema.get_as_req(intern!(schema.py(), "schema_ref"))?;
-        let serializer_id = definitions.get_reference_id(&schema_ref);
-        Ok(Self { serializer_id }.into())
+        let schema_ref = schema.get_as_req(intern!(schema.py(), "schema_ref"))?;
+        let definition = definitions.get_definition(schema_ref);
+        Ok(Self { definition }.into())
     }
 }
 
@@ -70,15 +68,17 @@ impl TypeSerializer for DefinitionRefSerializer {
         exclude: Option<&PyAny>,
         extra: &Extra,
     ) -> PyResult<PyObject> {
-        let value_id = extra.rec_guard.add(value, self.serializer_id)?;
-        let comb_serializer = extra.definitions.get(self.serializer_id).unwrap();
-        let r = comb_serializer.to_python(value, include, exclude, extra);
-        extra.rec_guard.pop(value_id, self.serializer_id);
-        r
+        self.definition.read(|comb_serializer| {
+            let comb_serializer = comb_serializer.unwrap();
+            let value_id = extra.rec_guard.add(value, self.definition.id())?;
+            let r = comb_serializer.to_python(value, include, exclude, extra);
+            extra.rec_guard.pop(value_id, self.definition.id());
+            r
+        })
     }
 
     fn json_key<'py>(&self, key: &'py PyAny, extra: &Extra) -> PyResult<Cow<'py, str>> {
-        self._invalid_as_json_key(key, extra, Self::EXPECTED_TYPE)
+        self.definition.read(|s| s.unwrap().json_key(key, extra))
     }
 
     fn serde_serialize<S: serde::ser::Serializer>(
@@ -89,14 +89,23 @@ impl TypeSerializer for DefinitionRefSerializer {
         exclude: Option<&PyAny>,
         extra: &Extra,
     ) -> Result<S::Ok, S::Error> {
-        let value_id = extra.rec_guard.add(value, self.serializer_id).map_err(py_err_se_err)?;
-        let comb_serializer = extra.definitions.get(self.serializer_id).unwrap();
-        let r = comb_serializer.serde_serialize(value, serializer, include, exclude, extra);
-        extra.rec_guard.pop(value_id, self.serializer_id);
-        r
+        self.definition.read(|comb_serializer| {
+            let comb_serializer = comb_serializer.unwrap();
+            let value_id = extra
+                .rec_guard
+                .add(value, self.definition.id())
+                .map_err(py_err_se_err)?;
+            let r = comb_serializer.serde_serialize(value, serializer, include, exclude, extra);
+            extra.rec_guard.pop(value_id, self.definition.id());
+            r
+        })
     }
 
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
+
+    fn retry_with_lax_check(&self) -> bool {
+        self.definition.read(|s| s.unwrap().retry_with_lax_check())
+    }
 }
