@@ -1,10 +1,11 @@
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
+
 use std::borrow::Cow;
 
 use serde::Serialize;
 
-use crate::definitions::DefinitionsBuilder;
+use crate::{definitions::DefinitionsBuilder, input::Int};
 
 use super::{
     infer_json_key, infer_serialize, infer_to_python, BuildSerializer, CombinedSerializer, Extra, IsType, ObType,
@@ -30,6 +31,8 @@ pub(crate) fn none_json_key() -> PyResult<Cow<'static, str>> {
     Ok(Cow::Borrowed("None"))
 }
 
+impl_py_gc_traverse!(NoneSerializer {});
+
 impl TypeSerializer for NoneSerializer {
     fn to_python(
         &self,
@@ -98,6 +101,8 @@ macro_rules! build_simple_serializer {
             }
         }
 
+        impl_py_gc_traverse!($struct_name {});
+
         impl TypeSerializer for $struct_name {
             fn to_python(
                 &self,
@@ -163,7 +168,7 @@ pub(crate) fn to_str_json_key(key: &PyAny) -> PyResult<Cow<str>> {
     Ok(key.str()?.to_string_lossy())
 }
 
-build_simple_serializer!(IntSerializer, "int", i64, ObType::Int, to_str_json_key);
+build_simple_serializer!(IntSerializer, "int", Int, ObType::Int, to_str_json_key);
 
 pub(crate) fn bool_json_key(key: &PyAny) -> PyResult<Cow<str>> {
     let v = if key.is_true().unwrap_or(false) {
@@ -175,4 +180,3 @@ pub(crate) fn bool_json_key(key: &PyAny) -> PyResult<Cow<str>> {
 }
 
 build_simple_serializer!(BoolSerializer, "bool", bool, ObType::Bool, bool_json_key);
-build_simple_serializer!(FloatSerializer, "float", f64, ObType::Float, to_str_json_key);
