@@ -4,6 +4,7 @@ use pyo3::types::PyDict;
 use crate::errors::{ErrorTypeDefaults, ValError, ValResult};
 use crate::input::Input;
 
+use super::validation_state::Exactness;
 use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
 #[derive(Debug, Clone)]
@@ -28,27 +29,16 @@ impl Validator for CallableValidator {
         &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
-        _state: &mut ValidationState,
+        state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
+        state.floor_exactness(Exactness::Lax);
         match input.callable() {
             true => Ok(input.to_object(py)),
             false => Err(ValError::new(ErrorTypeDefaults::CallableType, input)),
         }
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        _ultra_strict: bool,
-    ) -> bool {
-        false
-    }
-
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
