@@ -7,19 +7,20 @@ use ahash::AHashSet;
 
 use crate::build_tools::py_schema_err;
 use crate::build_tools::{is_strict, schema_or_config_same, ExtraBehavior};
-use crate::errors::{ErrorType, ErrorTypeDefaults, ValError, ValLineError, ValResult};
-use crate::input::{GenericArguments, Input};
+use crate::errors::{AsLocItem, ErrorType, ErrorTypeDefaults, ValError, ValLineError, ValResult};
+use crate::input::{BorrowInput, GenericArguments, Input, ValidationMatch};
 use crate::lookup_key::LookupKey;
 use crate::tools::SchemaDict;
 use crate::validators::function::convert_err;
 
 use super::arguments::{json_get, json_slice, py_get, py_slice};
 use super::model::{create_class, force_setattr, Revalidate};
+use super::validation_state::Exactness;
 use super::{
     build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, Extra, ValidationState, Validator,
 };
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 struct Field {
     kw_only: bool,
     name: String,
@@ -30,7 +31,7 @@ struct Field {
     frozen: bool,
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct DataclassArgsValidator {
     fields: Vec<Field>,
     positional_count: usize,
@@ -38,6 +39,7 @@ pub struct DataclassArgsValidator {
     dataclass_name: String,
     validator_name: String,
     extra_behavior: ExtraBehavior,
+    extras_validator: Option<Box<CombinedValidator>>,
     loc_by_alias: bool,
 }
 
@@ -55,6 +57,12 @@ impl BuildValidator for DataclassArgsValidator {
 
         let extra_behavior = ExtraBehavior::from_schema_or_config(py, schema, config, ExtraBehavior::Ignore)?;
 
+        let extras_validator = match (schema.get_item(intern!(py, "extras_schema"))?, &extra_behavior) {
+            (Some(v), ExtraBehavior::Allow) => Some(Box::new(build_validator(v, config, definitions)?)),
+            (Some(_), _) => return py_schema_err!("extras_schema can only be used if extra_behavior=allow"),
+            (_, _) => None,
+        };
+
         let fields_schema: &PyList = schema.get_as_req(intern!(py, "fields"))?;
         let mut fields: Vec<Field> = Vec::with_capacity(fields_schema.len());
 
@@ -66,7 +74,7 @@ impl BuildValidator for DataclassArgsValidator {
             let py_name: &PyString = field.get_as_req(intern!(py, "name"))?;
             let name: String = py_name.extract()?;
 
-            let lookup_key = match field.get_item(intern!(py, "validation_alias")) {
+            let lookup_key = match field.get_item(intern!(py, "validation_alias"))? {
                 Some(alias) => {
                     let alt_alias = if populate_by_name { Some(name.as_str()) } else { None };
                     LookupKey::from_py(py, alias, alt_alias)?
@@ -118,6 +126,7 @@ impl BuildValidator for DataclassArgsValidator {
             dataclass_name,
             validator_name,
             extra_behavior,
+            extras_validator,
             loc_by_alias: config.get_as(intern!(py, "loc_by_alias"))?.unwrap_or(true),
         }
         .into())
@@ -180,15 +189,21 @@ impl Validator for DataclassArgsValidator {
                                     kw_value = Some((lookup_path, value));
                                 }
                             }
+                            let kw_value = kw_value
+                                .as_ref()
+                                .map(|(path, value)| (path, value.borrow_input()));
 
                             match (pos_value, kw_value) {
                                 // found both positional and keyword arguments, error
                                 (Some(_), Some((_, kw_value))) => {
-                                    errors.push(ValLineError::new_with_loc(
-                                        ErrorTypeDefaults::MultipleArgumentValues,
-                                        kw_value,
-                                        field.name.clone(),
-                                    ));
+                                    errors.push(
+                                        ValLineError::new_with_loc(
+                                            ErrorTypeDefaults::MultipleArgumentValues,
+                                            kw_value,
+                                            field.name.clone(),
+                                        )
+                                        .into_owned(py),
+                                    );
                                 }
                                 // found a positional argument, validate it
                                 (Some(pos_value), None) => match field.validator.validate(py, pos_value, state) {
@@ -208,27 +223,41 @@ impl Validator for DataclassArgsValidator {
                                         Ok(value) => set_item!(field, value),
                                         Err(ValError::LineErrors(line_errors)) => {
                                             errors.extend(line_errors.into_iter().map(|err| {
-                                                lookup_path.apply_error_loc(err, self.loc_by_alias, &field.name)
+                                                lookup_path
+                                                    .apply_error_loc(err, self.loc_by_alias, &field.name)
+                                                    .into_owned(py)
                                             }));
                                         }
-                                        Err(err) => return Err(err),
+                                        Err(err) => return Err(err.into_owned(py)),
                                     }
                                 }
                                 // found neither, check if there is a default value, otherwise error
                                 (None, None) => {
-                                    if let Some(value) =
-                                        field
-                                            .validator
-                                            .default_value(py, Some(field.name.as_str()), state)?
-                                    {
-                                        set_item!(field, value);
-                                    } else {
-                                        errors.push(field.lookup_key.error(
-                                            ErrorTypeDefaults::Missing,
-                                            input,
-                                            self.loc_by_alias,
-                                            &field.name,
-                                        ));
+                                    match field.validator.default_value(py, Some(field.name.as_str()), state) {
+                                        Ok(Some(value)) => {
+                                            // Default value exists, and passed validation if required
+                                            set_item!(field, value);
+                                        },
+                                        Ok(None) => {
+                                            // This means there was no default value
+                                            errors.push(field.lookup_key.error(
+                                                ErrorTypeDefaults::Missing,
+                                                input,
+                                                self.loc_by_alias,
+                                                &field.name
+                                            ));
+                                        },
+                                        Err(ValError::Omit) => continue,
+                                        Err(ValError::LineErrors(line_errors)) => {
+                                            for err in line_errors {
+                                                // Note: this will always use the field name even if there is an alias
+                                                // However, we don't mind so much because this error can only happen if the
+                                                // default value fails validation, which is arguably a developer error.
+                                                // We could try to "fix" this in the future if desired.
+                                                errors.push(err);
+                                            }
+                                        }
+                                        Err(err) => return Err(err),
                                     }
                                 }
                             }
@@ -253,21 +282,39 @@ impl Validator for DataclassArgsValidator {
                         if let Some(kwargs) = $args.kwargs {
                             if kwargs.len() != used_keys.len() {
                                 for (raw_key, value) in kwargs.iter() {
-                                    match raw_key.strict_str() {
+                                    match raw_key.validate_str(true, false).map(ValidationMatch::into_inner) {
                                         Ok(either_str) => {
                                             if !used_keys.contains(either_str.as_cow()?.as_ref()) {
                                                 // Unknown / extra field
                                                 match self.extra_behavior {
                                                     ExtraBehavior::Forbid => {
-                                                        errors.push(ValLineError::new_with_loc(
-                                                            ErrorTypeDefaults::UnexpectedKeywordArgument,
-                                                            value,
-                                                            raw_key.as_loc_item(),
-                                                        ));
+                                                        errors.push(
+                                                            ValLineError::new_with_loc(
+                                                                ErrorTypeDefaults::UnexpectedKeywordArgument,
+                                                                value,
+                                                                raw_key.as_loc_item(),
+                                                            )
+                                                            .into_owned(py),
+                                                        );
                                                     }
                                                     ExtraBehavior::Ignore => {}
                                                     ExtraBehavior::Allow => {
-                                                        output_dict.set_item(either_str.as_py_string(py), value)?
+                                                        if let Some(ref validator) = self.extras_validator {
+                                                            match validator.validate(py, value, state) {
+                                                                Ok(value) => output_dict
+                                                                    .set_item(either_str.as_py_string(py), value)?,
+                                                                Err(ValError::LineErrors(line_errors)) => {
+                                                                    for err in line_errors {
+                                                                        errors.push(err.with_outer_location(
+                                                                            raw_key.as_loc_item(),
+                                                                        ));
+                                                                    }
+                                                                }
+                                                                Err(err) => return Err(err),
+                                                            }
+                                                        } else {
+                                                            output_dict.set_item(either_str.as_py_string(py), value)?
+                                                        }
                                                     }
                                                 }
                                             }
@@ -287,9 +334,24 @@ impl Validator for DataclassArgsValidator {
                         }
                     }};
                 }
+
                 match args {
                     GenericArguments::Py(a) => process!(a, py_get_dict_item, py_get, py_slice),
                     GenericArguments::Json(a) => process!(a, json_get, json_get, json_slice),
+                    GenericArguments::StringMapping(a) => {
+                        // StringMapping cannot pass positional args, so wrap the PyDict
+                        // in a type with guaranteed empty args array for sake of the process
+                        // macro
+                        struct StringMappingArgs<'a> {
+                            args: Option<&'a PyTuple>,
+                            kwargs: Option<&'a PyDict>,
+                        }
+                        let a = StringMappingArgs {
+                            args: None,
+                            kwargs: Some(a),
+                        };
+                        process!(a, py_get_string_mapping_item, py_get, py_slice);
+                    }
                 }
                 Ok(())
             },
@@ -377,28 +439,12 @@ impl Validator for DataclassArgsValidator {
         }
     }
 
-    fn different_strict_behavior(
-        &self,
-        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        self.fields
-            .iter()
-            .any(|f| f.validator.different_strict_behavior(definitions, ultra_strict))
-    }
-
     fn get_name(&self) -> &str {
         &self.validator_name
     }
-
-    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        self.fields
-            .iter_mut()
-            .try_for_each(|field| field.validator.complete(definitions))
-    }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct DataclassValidator {
     strict: bool,
     validator: Box<CombinedValidator>,
@@ -499,6 +545,7 @@ impl Validator for DataclassValidator {
             ))
         } else {
             let val_output = self.validator.validate(py, input, state)?;
+            state.floor_exactness(Exactness::Strict);
             let dc = create_class(self.class.as_ref(py))?;
             self.set_dict_call(py, dc.as_ref(py), val_output, input)?;
             Ok(dc)
@@ -529,7 +576,7 @@ impl Validator for DataclassValidator {
 
         if self.slots {
             let value = dc_dict
-                .get_item(field_name)
+                .get_item(field_name)?
                 .ok_or_else(|| PyKeyError::new_err(field_name.to_string()))?;
             force_setattr(py, obj, field_name, value)?;
         } else {
@@ -539,25 +586,9 @@ impl Validator for DataclassValidator {
         Ok(obj.to_object(py))
     }
 
-    fn different_strict_behavior(
-        &self,
-        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        if ultra_strict {
-            self.validator.different_strict_behavior(definitions, ultra_strict)
-        } else {
-            true
-        }
-    }
-
     fn get_name(&self) -> &str {
         &self.name
     }
-
-    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        self.validator.complete(definitions)
-    }
 }
 
 impl DataclassValidator {
