@@ -65,7 +65,9 @@ impl Validator for DateTimeValidator {
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let strict = state.strict_or(self.strict);
-        let datetime = input.validate_datetime(strict, self.microseconds_precision)?;
+        let datetime = input
+            .validate_datetime(strict, self.microseconds_precision)?
+            .unpack(state);
         if let Some(constraints) = &self.constraints {
             // if we get an error from as_speedate, it's probably because the input datetime was invalid
             // specifically had an invalid tzinfo, hence here we return a validation error
@@ -125,21 +127,9 @@ impl Validator for DateTimeValidator {
         Ok(datetime.try_into_py(py)?)
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        !ultra_strict
-    }
-
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 #[derive(Debug, Clone)]
@@ -260,7 +250,7 @@ impl TZConstraint {
 
     pub(super) fn from_py(schema: &PyDict) -> PyResult<Option<Self>> {
         let py = schema.py();
-        let tz_constraint = match schema.get_item(intern!(py, "tz_constraint")) {
+        let tz_constraint = match schema.get_item(intern!(py, "tz_constraint"))? {
             Some(c) => c,
             None => return Ok(None),
         };
