@@ -1,3 +1,5 @@
+use std::cmp::Ordering;
+
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
@@ -19,11 +21,11 @@ impl BuildValidator for FloatBuilder {
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
-        let use_constrained = schema.get_item(intern!(py, "multiple_of")).is_some()
-            || schema.get_item(intern!(py, "le")).is_some()
-            || schema.get_item(intern!(py, "lt")).is_some()
-            || schema.get_item(intern!(py, "ge")).is_some()
-            || schema.get_item(intern!(py, "gt")).is_some();
+        let use_constrained = schema.get_item(intern!(py, "multiple_of"))?.is_some()
+            || schema.get_item(intern!(py, "le"))?.is_some()
+            || schema.get_item(intern!(py, "lt"))?.is_some()
+            || schema.get_item(intern!(py, "ge"))?.is_some()
+            || schema.get_item(intern!(py, "gt"))?.is_some();
         if use_constrained {
             ConstrainedFloatValidator::build(schema, config, definitions)
         } else {
@@ -68,29 +70,16 @@ impl Validator for FloatValidator {
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let strict = state.strict_or(self.strict);
-        let either_float = input.validate_float(strict, state.extra().ultra_strict)?;
+        let either_float = input.validate_float(state.strict_or(self.strict))?.unpack(state);
         if !self.allow_inf_nan && !either_float.as_f64().is_finite() {
             return Err(ValError::new(ErrorTypeDefaults::FiniteNumber, input));
         }
         Ok(either_float.into_py(py))
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        _ultra_strict: bool,
-    ) -> bool {
-        true
-    }
-
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 #[derive(Debug, Clone)]
@@ -113,15 +102,14 @@ impl Validator for ConstrainedFloatValidator {
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let strict = state.strict_or(self.strict);
-        let either_float = input.validate_float(strict, state.extra().ultra_strict)?;
+        let either_float = input.validate_float(state.strict_or(self.strict))?.unpack(state);
         let float: f64 = either_float.as_f64();
         if !self.allow_inf_nan && !float.is_finite() {
             return Err(ValError::new(ErrorTypeDefaults::FiniteNumber, input));
         }
         if let Some(multiple_of) = self.multiple_of {
             let rem = float % multiple_of;
-            let threshold = float / 1e9;
+            let threshold = float.abs() / 1e9;
             if rem.abs() > threshold && (rem - multiple_of).abs() > threshold {
                 return Err(ValError::new(
                     ErrorType::MultipleOf {
@@ -133,7 +121,7 @@ impl Validator for ConstrainedFloatValidator {
             }
         }
         if let Some(le) = self.le {
-            if float > le {
+            if !matches!(float.partial_cmp(&le), Some(Ordering::Less | Ordering::Equal)) {
                 return Err(ValError::new(
                     ErrorType::LessThanEqual {
                         le: le.into(),
@@ -144,7 +132,7 @@ impl Validator for ConstrainedFloatValidator {
             }
         }
         if let Some(lt) = self.lt {
-            if float >= lt {
+            if !matches!(float.partial_cmp(&lt), Some(Ordering::Less)) {
                 return Err(ValError::new(
                     ErrorType::LessThan {
                         lt: lt.into(),
@@ -155,7 +143,7 @@ impl Validator for ConstrainedFloatValidator {
             }
         }
         if let Some(ge) = self.ge {
-            if float < ge {
+            if !matches!(float.partial_cmp(&ge), Some(Ordering::Greater | Ordering::Equal)) {
                 return Err(ValError::new(
                     ErrorType::GreaterThanEqual {
                         ge: ge.into(),
@@ -166,7 +154,7 @@ impl Validator for ConstrainedFloatValidator {
             }
         }
         if let Some(gt) = self.gt {
-            if float <= gt {
+            if !matches!(float.partial_cmp(&gt), Some(Ordering::Greater)) {
                 return Err(ValError::new(
                     ErrorType::GreaterThan {
                         gt: gt.into(),
@@ -179,21 +167,9 @@ impl Validator for ConstrainedFloatValidator {
         Ok(either_float.into_py(py))
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        _ultra_strict: bool,
-    ) -> bool {
-        true
-    }
-
     fn get_name(&self) -> &str {
         "constrained-float"
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 impl BuildValidator for ConstrainedFloatValidator {
