@@ -8,8 +8,7 @@ use crate::errors::{ErrorType, ValError, ValResult};
 use crate::input::{Input, Int};
 use crate::tools::SchemaDict;
 
-use super::ValidationState;
-use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, Validator};
+use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
 #[derive(Debug, Clone)]
 pub struct IntValidator {
@@ -25,11 +24,11 @@ impl BuildValidator for IntValidator {
         _definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
-        let use_constrained = schema.get_item(intern!(py, "multiple_of")).is_some()
-            || schema.get_item(intern!(py, "le")).is_some()
-            || schema.get_item(intern!(py, "lt")).is_some()
-            || schema.get_item(intern!(py, "ge")).is_some()
-            || schema.get_item(intern!(py, "gt")).is_some();
+        let use_constrained = schema.get_item(intern!(py, "multiple_of"))?.is_some()
+            || schema.get_item(intern!(py, "le"))?.is_some()
+            || schema.get_item(intern!(py, "lt"))?.is_some()
+            || schema.get_item(intern!(py, "ge"))?.is_some()
+            || schema.get_item(intern!(py, "gt"))?.is_some();
         if use_constrained {
             ConstrainedIntValidator::build(schema, config)
         } else {
@@ -50,25 +49,14 @@ impl Validator for IntValidator {
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let either_int = input.validate_int(state.strict_or(self.strict))?;
-        Ok(either_int.into_py(py))
-    }
-
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        !ultra_strict
+        input
+            .validate_int(state.strict_or(self.strict))
+            .map(|val_match| val_match.unpack(state).into_py(py))
     }
 
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 #[derive(Debug, Clone)]
@@ -90,7 +78,7 @@ impl Validator for ConstrainedIntValidator {
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let either_int = input.validate_int(state.strict_or(self.strict))?;
+        let either_int = input.validate_int(state.strict_or(self.strict))?.unpack(state);
         let int_value = either_int.as_int()?;
 
         if let Some(ref multiple_of) = self.multiple_of {
@@ -151,21 +139,9 @@ impl Validator for ConstrainedIntValidator {
         Ok(either_int.into_py(py))
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        !ultra_strict
-    }
-
     fn get_name(&self) -> &str {
         "constrained-int"
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 impl ConstrainedIntValidator {
