@@ -7,11 +7,9 @@ use crate::errors::ValResult;
 use crate::input::Input;
 use crate::tools::SchemaDict;
 
-use super::InputType;
-use super::ValidationState;
-use super::{build_validator, BuildValidator, CombinedValidator, Validator};
+use super::{build_validator, BuildValidator, CombinedValidator, InputType, ValidationState, Validator};
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct JsonOrPython {
     json: Box<CombinedValidator>,
     python: Box<CombinedValidator>,
@@ -57,27 +55,13 @@ impl Validator for JsonOrPython {
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        match state.extra().mode {
+        match state.extra().input_type {
             InputType::Python => self.python.validate(py, input, state),
-            InputType::Json => self.json.validate(py, input, state),
+            _ => self.json.validate(py, input, state),
         }
     }
 
-    fn different_strict_behavior(
-        &self,
-        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        self.json.different_strict_behavior(definitions, ultra_strict)
-            || self.python.different_strict_behavior(definitions, ultra_strict)
-    }
-
     fn get_name(&self) -> &str {
         &self.name
     }
-
-    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        self.json.complete(definitions)?;
-        self.python.complete(definitions)
-    }
 }
