@@ -1,32 +1,35 @@
+use std::sync::OnceLock;
+
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
 
 use crate::errors::ValResult;
 use crate::input::{GenericIterable, Input};
 use crate::tools::SchemaDict;
+use crate::validators::Exactness;
 
 use super::{build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct ListValidator {
     strict: bool,
     item_validator: Option<Box<CombinedValidator>>,
     min_length: Option<usize>,
     max_length: Option<usize>,
-    name: String,
+    name: OnceLock<String>,
 }
 
 pub fn get_items_schema(
     schema: &PyDict,
     config: Option<&PyDict>,
     definitions: &mut DefinitionsBuilder<CombinedValidator>,
-) -> PyResult<Option<Box<CombinedValidator>>> {
-    match schema.get_item(pyo3::intern!(schema.py(), "items_schema")) {
+) -> PyResult<Option<CombinedValidator>> {
+    match schema.get_item(pyo3::intern!(schema.py(), "items_schema"))? {
         Some(d) => {
             let validator = build_validator(d, config, definitions)?;
             match validator {
                 CombinedValidator::Any(_) => Ok(None),
-                _ => Ok(Some(Box::new(validator))),
+                _ => Ok(Some(validator)),
             }
         }
         None => Ok(None),
@@ -58,7 +61,7 @@ macro_rules! length_check {
                     crate::errors::ErrorType::TooLong {
                         field_type: $field_type.to_string(),
                         max_length,
-                        actual_length,
+                        actual_length: Some(actual_length),
                         context: None,
                     },
                     $input,
@@ -98,15 +101,13 @@ impl BuildValidator for ListValidator {
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
-        let item_validator = get_items_schema(schema, config, definitions)?;
-        let inner_name = item_validator.as_ref().map_or("any", |v| v.get_name());
-        let name = format!("{}[{inner_name}]", Self::EXPECTED_TYPE);
+        let item_validator = get_items_schema(schema, config, definitions)?.map(Box::new);
         Ok(Self {
             strict: crate::build_tools::is_strict(schema, config)?,
             item_validator,
             min_length: schema.get_as(pyo3::intern!(py, "min_length"))?,
             max_length: schema.get_as(pyo3::intern!(py, "max_length"))?,
-            name,
+            name: OnceLock::new(),
         }
         .into())
     }
@@ -122,6 +123,12 @@ impl Validator for ListValidator {
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let seq = input.validate_list(state.strict_or(self.strict))?;
+        let exactness = match &seq {
+            GenericIterable::List(_) | GenericIterable::JsonArray(_) => Exactness::Exact,
+            GenericIterable::Tuple(_) => Exactness::Strict,
+            _ => Exactness::Lax,
+        };
+        state.floor_exactness(exactness);
 
         let output = match self.item_validator {
             Some(ref v) => seq.validate_to_vec(py, input, self.max_length, "List", v, state)?,
@@ -138,31 +145,22 @@ impl Validator for ListValidator {
         Ok(output.into_py(py))
     }
 
-    fn different_strict_behavior(
-        &self,
-        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        if ultra_strict {
-            match self.item_validator {
-                Some(ref v) => v.different_strict_behavior(definitions, true),
-                None => false,
-            }
-        } else {
-            true
-        }
-    }
-
     fn get_name(&self) -> &str {
-        &self.name
-    }
-
-    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        if let Some(ref mut v) = self.item_validator {
-            v.complete(definitions)?;
-            let inner_name = v.get_name();
-            self.name = format!("{}[{inner_name}]", Self::EXPECTED_TYPE);
+        // The logic here is a little janky, it's done to try to cache the formatted name
+        // while also trying to render definitions correctly when possible.
+        //
+        // Probably an opportunity for a future refactor
+        match self.name.get() {
+            Some(s) => s.as_str(),
+            None => {
+                let name = self.item_validator.as_ref().map_or("any", |v| v.get_name());
+                if name == "..." {
+                    // when inner name is not initialized yet, don't cache it here
+                    "list[...]"
+                } else {
+                    self.name.get_or_init(|| format!("list[{name}]")).as_str()
+                }
+            }
         }
-        Ok(())
     }
 }
