@@ -2,13 +2,14 @@ use pyo3::prelude::*;
 use pyo3::types::{PyDict, PySet};
 
 use crate::errors::ValResult;
-use crate::input::Input;
+use crate::input::{GenericIterable, Input};
 use crate::tools::SchemaDict;
+use crate::validators::Exactness;
 
 use super::list::min_length_check;
 use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct SetValidator {
     strict: bool,
     item_validator: Box<CombinedValidator>,
@@ -25,7 +26,7 @@ macro_rules! set_build {
             definitions: &mut DefinitionsBuilder<CombinedValidator>,
         ) -> PyResult<CombinedValidator> {
             let py = schema.py();
-            let item_validator = match schema.get_item(pyo3::intern!(schema.py(), "items_schema")) {
+            let item_validator = match schema.get_item(pyo3::intern!(schema.py(), "items_schema"))? {
                 Some(d) => Box::new(crate::validators::build_validator(d, config, definitions)?),
                 None => Box::new(crate::validators::any::AnyValidator::build(
                     schema,
@@ -64,29 +65,19 @@ impl Validator for SetValidator {
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         let collection = input.validate_set(state.strict_or(self.strict))?;
+        let exactness = match &collection {
+            GenericIterable::Set(_) => Exactness::Exact,
+            GenericIterable::FrozenSet(_) | GenericIterable::JsonArray(_) => Exactness::Strict,
+            _ => Exactness::Lax,
+        };
+        state.floor_exactness(exactness);
         let set = PySet::empty(py)?;
         collection.validate_to_set(py, set, input, self.max_length, "Set", &self.item_validator, state)?;
         min_length_check!(input, "Set", self.min_length, set);
         Ok(set.into_py(py))
     }
 
-    fn different_strict_behavior(
-        &self,
-        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        if ultra_strict {
-            self.item_validator.different_strict_behavior(definitions, true)
-        } else {
-            true
-        }
-    }
-
     fn get_name(&self) -> &str {
         &self.name
     }
-
-    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        self.item_validator.complete(definitions)
-    }
 }
