@@ -10,9 +10,10 @@ use crate::tools::SchemaDict;
 
 use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct StrValidator {
     strict: bool,
+    coerce_numbers_to_str: bool,
 }
 
 impl BuildValidator for StrValidator {
@@ -30,6 +31,7 @@ impl BuildValidator for StrValidator {
         } else {
             Ok(Self {
                 strict: con_str_validator.strict,
+                coerce_numbers_to_str: con_str_validator.coerce_numbers_to_str,
             }
             .into())
         }
@@ -45,37 +47,27 @@ impl Validator for StrValidator {
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let either_str = input.validate_str(state.strict_or(self.strict))?;
-        Ok(either_str.into_py(py))
-    }
-
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        !ultra_strict
+        input
+            .validate_str(state.strict_or(self.strict), self.coerce_numbers_to_str)
+            .map(|val_match| val_match.unpack(state).into_py(py))
     }
 
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 /// Any new properties set here must be reflected in `has_constraints_set`
 #[derive(Debug, Clone, Default)]
 pub struct StrConstrainedValidator {
     strict: bool,
-    pattern: Option<Regex>,
+    pattern: Option<Pattern>,
     max_length: Option<usize>,
     min_length: Option<usize>,
     strip_whitespace: bool,
     to_lower: bool,
     to_upper: bool,
+    coerce_numbers_to_str: bool,
 }
 
 impl_py_gc_traverse!(StrConstrainedValidator {});
@@ -87,7 +79,9 @@ impl Validator for StrConstrainedValidator {
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let either_str = input.validate_str(state.strict_or(self.strict))?;
+        let either_str = input
+            .validate_str(state.strict_or(self.strict), self.coerce_numbers_to_str)?
+            .unpack(state);
         let cow = either_str.as_cow()?;
         let mut str = cow.as_ref();
         if self.strip_whitespace {
@@ -123,10 +117,10 @@ impl Validator for StrConstrainedValidator {
         }
 
         if let Some(pattern) = &self.pattern {
-            if !pattern.is_match(str) {
+            if !pattern.is_match(py, str)? {
                 return Err(ValError::new(
                     ErrorType::StringPatternMismatch {
-                        pattern: pattern.to_string(),
+                        pattern: pattern.pattern.clone(),
                         context: None,
                     },
                     input,
@@ -147,30 +141,24 @@ impl Validator for StrConstrainedValidator {
         Ok(py_string.into_py(py))
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        !ultra_strict
-    }
-
     fn get_name(&self) -> &str {
         "constrained-str"
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 impl StrConstrainedValidator {
     fn build(schema: &PyDict, config: Option<&PyDict>) -> PyResult<Self> {
         let py = schema.py();
-        let pattern = match schema.get_as(intern!(py, "pattern"))? {
-            Some(s) => Some(Regex::new(s).map_err(|e| py_schema_error_type!("{}", e))?),
-            None => None,
-        };
+
+        let pattern = schema
+            .get_as(intern!(py, "pattern"))?
+            .map(|s| {
+                let regex_engine =
+                    schema_or_config(schema, config, intern!(py, "regex_engine"), intern!(py, "regex_engine"))?
+                        .unwrap_or(RegexEngine::RUST_REGEX);
+                Pattern::compile(py, s, regex_engine)
+            })
+            .transpose()?;
         let min_length: Option<usize> =
             schema_or_config(schema, config, intern!(py, "min_length"), intern!(py, "str_min_length"))?;
         let max_length: Option<usize> =
@@ -188,6 +176,11 @@ impl StrConstrainedValidator {
         let to_upper: bool =
             schema_or_config(schema, config, intern!(py, "to_upper"), intern!(py, "str_to_upper"))?.unwrap_or(false);
 
+        let coerce_numbers_to_str = match config {
+            Some(c) => c.get_item("coerce_numbers_to_str")?.map_or(Ok(false), PyAny::is_true)?,
+            None => false,
+        };
+
         Ok(Self {
             strict: is_strict(schema, config)?,
             pattern,
@@ -196,6 +189,7 @@ impl StrConstrainedValidator {
             strip_whitespace,
             to_lower,
             to_upper,
+            coerce_numbers_to_str,
         })
     }
 
@@ -210,3 +204,45 @@ impl StrConstrainedValidator {
             || self.to_upper
     }
 }
+
+#[derive(Debug, Clone)]
+struct Pattern {
+    pattern: String,
+    engine: RegexEngine,
+}
+
+#[derive(Debug, Clone)]
+enum RegexEngine {
+    RustRegex(Regex),
+    PythonRe(PyObject),
+}
+
+impl RegexEngine {
+    const RUST_REGEX: &'static str = "rust-regex";
+    const PYTHON_RE: &'static str = "python-re";
+}
+
+impl Pattern {
+    fn compile(py: Python<'_>, pattern: String, engine: &str) -> PyResult<Self> {
+        let engine = match engine {
+            RegexEngine::RUST_REGEX => {
+                RegexEngine::RustRegex(Regex::new(&pattern).map_err(|e| py_schema_error_type!("{}", e))?)
+            }
+            RegexEngine::PYTHON_RE => {
+                let re_compile = py.import(intern!(py, "re"))?.getattr(intern!(py, "compile"))?;
+                RegexEngine::PythonRe(re_compile.call1((&pattern,))?.into())
+            }
+            _ => return Err(py_schema_error_type!("Invalid regex engine: {}", engine)),
+        };
+        Ok(Self { pattern, engine })
+    }
+
+    fn is_match(&self, py: Python<'_>, target: &str) -> PyResult<bool> {
+        match &self.engine {
+            RegexEngine::RustRegex(regex) => Ok(regex.is_match(target)),
+            RegexEngine::PythonRe(py_regex) => {
+                Ok(!py_regex.call_method1(py, intern!(py, "match"), (target,))?.is_none(py))
+            }
+        }
+    }
+}
