@@ -1,5 +1,5 @@
 use pyo3::prelude::*;
-use pyo3::types::PyDict;
+use pyo3::types::{PyDelta, PyDeltaAccess, PyDict};
 use speedate::Duration;
 
 use crate::build_tools::is_strict;
@@ -25,7 +25,7 @@ struct TimedeltaConstraints {
 }
 
 fn get_constraint(schema: &PyDict, key: &str) -> PyResult<Option<Duration>> {
-    match schema.get_item(key) {
+    match schema.get_item(key)? {
         Some(value) => {
             let either_timedelta = EitherTimedelta::try_from(value)?;
             Ok(Some(either_timedelta.to_duration()?))
@@ -71,7 +71,9 @@ impl Validator for TimeDeltaValidator {
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
-        let timedelta = input.validate_timedelta(state.strict_or(self.strict), self.microseconds_precision)?;
+        let timedelta = input
+            .validate_timedelta(state.strict_or(self.strict), self.microseconds_precision)?
+            .unpack(state);
         let py_timedelta = timedelta.try_into_py(py)?;
         if let Some(constraints) = &self.constraints {
             let raw_timedelta = timedelta.to_duration()?;
@@ -83,9 +85,7 @@ impl Validator for TimeDeltaValidator {
                             return Err(ValError::new(
                                 ErrorType::$error {
                                     context: None,
-                                    $constraint: duration_as_pytimedelta(py, constraint)?
-                                        .repr()?
-                                        .to_string()
+                                    $constraint: pydelta_to_human_readable(duration_as_pytimedelta(py, constraint)?)
                                         .into(),
                                 },
                                 py_timedelta.as_ref(),
@@ -103,19 +103,47 @@ impl Validator for TimeDeltaValidator {
         Ok(py_timedelta.into())
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        !ultra_strict
-    }
-
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
+}
+fn pydelta_to_human_readable(py_delta: &PyDelta) -> String {
+    let total_seconds = py_delta.get_seconds();
+    let hours = total_seconds / 3600;
+    let minutes = (total_seconds % 3600) / 60;
+    let seconds = total_seconds % 60;
+    let microseconds = py_delta.get_microseconds();
+    let days = py_delta.get_days();
 
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
+    let mut formatted_duration = Vec::new();
+
+    if days != 0 {
+        formatted_duration.push(format!("{} day{}", days, if days != 1 { "s" } else { "" }));
+    }
+
+    if hours != 0 {
+        formatted_duration.push(format!("{} hour{}", hours, if hours != 1 { "s" } else { "" }));
+    }
+
+    if minutes != 0 {
+        formatted_duration.push(format!("{} minute{}", minutes, if minutes != 1 { "s" } else { "" }));
+    }
+
+    if seconds != 0 {
+        formatted_duration.push(format!("{} second{}", seconds, if seconds != 1 { "s" } else { "" }));
+    }
+
+    if microseconds != 0 {
+        formatted_duration.push(format!(
+            "{} microsecond{}",
+            microseconds,
+            if microseconds != 1 { "s" } else { "" }
+        ));
     }
+
+    if formatted_duration.is_empty() {
+        formatted_duration.push("0 seconds".to_string());
+    }
+
+    formatted_duration.join(" and ")
 }
