@@ -1,3 +1,5 @@
+use std::ops::ControlFlow;
+
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyString};
@@ -6,10 +8,10 @@ use ahash::AHashSet;
 
 use crate::build_tools::py_schema_err;
 use crate::build_tools::{is_strict, schema_or_config, schema_or_config_same, ExtraBehavior};
-use crate::errors::{py_err_string, ErrorType, ErrorTypeDefaults, ValError, ValLineError, ValResult};
+use crate::errors::{AsLocItem, ErrorTypeDefaults, ValError, ValLineError, ValResult};
 use crate::input::{
-    AttributesGenericIterator, DictGenericIterator, GenericMapping, Input, JsonObjectGenericIterator,
-    MappingGenericIterator,
+    AttributesGenericIterator, BorrowInput, DictGenericIterator, GenericMapping, Input, JsonObjectGenericIterator,
+    MappingGenericIterator, StringMappingGenericIterator, ValidationMatch,
 };
 use crate::lookup_key::LookupKey;
 use crate::tools::SchemaDict;
@@ -18,9 +20,7 @@ use super::{
     build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, Extra, ValidationState, Validator,
 };
 
-use std::ops::ControlFlow;
-
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 struct TypedDictField {
     name: String,
     lookup_key: LookupKey,
@@ -31,11 +31,11 @@ struct TypedDictField {
 
 impl_py_gc_traverse!(TypedDictField { validator });
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct TypedDictValidator {
     fields: Vec<TypedDictField>,
     extra_behavior: ExtraBehavior,
-    extra_validator: Option<Box<CombinedValidator>>,
+    extras_validator: Option<Box<CombinedValidator>>,
     strict: bool,
     loc_by_alias: bool,
 }
@@ -61,9 +61,9 @@ impl BuildValidator for TypedDictValidator {
 
         let extra_behavior = ExtraBehavior::from_schema_or_config(py, schema, config, ExtraBehavior::Ignore)?;
 
-        let extra_validator = match (schema.get_item(intern!(py, "extra_validator")), &extra_behavior) {
+        let extras_validator = match (schema.get_item(intern!(py, "extras_schema"))?, &extra_behavior) {
             (Some(v), ExtraBehavior::Allow) => Some(Box::new(build_validator(v, config, definitions)?)),
-            (Some(_), _) => return py_schema_err!("extra_validator can only be used if extra_behavior=allow"),
+            (Some(_), _) => return py_schema_err!("extras_schema can only be used if extra_behavior=allow"),
             (_, _) => None,
         };
 
@@ -109,7 +109,7 @@ impl BuildValidator for TypedDictValidator {
                 }
             }
 
-            let lookup_key = match field_info.get_item(intern!(py, "validation_alias")) {
+            let lookup_key = match field_info.get_item(intern!(py, "validation_alias"))? {
                 Some(alias) => {
                     let alt_alias = if populate_by_name { Some(field_name) } else { None };
                     LookupKey::from_py(py, alias, alt_alias)?
@@ -129,7 +129,7 @@ impl BuildValidator for TypedDictValidator {
         Ok(Self {
             fields,
             extra_behavior,
-            extra_validator,
+            extras_validator,
             strict,
             loc_by_alias: config.get_as(intern!(py, "loc_by_alias"))?.unwrap_or(true),
         }
@@ -139,7 +139,7 @@ impl BuildValidator for TypedDictValidator {
 
 impl_py_gc_traverse!(TypedDictValidator {
     fields,
-    extra_validator
+    extras_validator
 });
 
 impl Validator for TypedDictValidator {
@@ -181,17 +181,13 @@ impl Validator for TypedDictValidator {
                     for field in &self.fields {
                         let op_key_value = match field.lookup_key.$get_method($dict $(, $kwargs )? ) {
                             Ok(v) => v,
-                            Err(err) => {
-                                errors.push(ValLineError::new_with_loc(
-                                    ErrorType::GetAttributeError {
-                                        error: py_err_string(py, err),
-                                        context: None,
-                                    },
-                                    input,
-                                    field.name.clone(),
-                                ));
+                            Err(ValError::LineErrors(line_errors)) => {
+                                for err in line_errors {
+                                    errors.push(err.with_outer_location(field.name.as_loc_item()));
+                                }
                                 continue;
                             }
+                            Err(err) => return ControlFlow::Break(err),
                         };
                         if let Some((lookup_path, value)) = op_key_value {
                             if let Some(ref mut used_keys) = used_keys {
@@ -199,28 +195,52 @@ impl Validator for TypedDictValidator {
                                 // extra logic either way
                                 used_keys.insert(lookup_path.first_key());
                             }
-                            match field.validator.validate(py, value, state) {
+                            match field.validator.validate(py, value.borrow_input(), state) {
                                 Ok(value) => {
                                     control_flow!(output_dict.set_item(&field.name_py, value))?;
                                 }
                                 Err(ValError::Omit) => continue,
                                 Err(ValError::LineErrors(line_errors)) => {
                                     for err in line_errors {
-                                        errors.push(lookup_path.apply_error_loc(err, self.loc_by_alias, &field.name));
+                                        errors.push(
+                                            lookup_path
+                                            .apply_error_loc(err, self.loc_by_alias, &field.name)
+                                            .into_owned(py)
+                                        );
                                     }
                                 }
-                                Err(err) => return ControlFlow::Break(err),
+                                Err(err) => return ControlFlow::Break(err.into_owned(py)),
                             }
                             continue;
-                        } else if let Some(value) = control_flow!(field.validator.default_value(py, Some(field.name.as_str()), state))? {
-                            control_flow!(output_dict.set_item(&field.name_py, value))?;
-                        } else if field.required {
-                            errors.push(field.lookup_key.error(
-                                ErrorTypeDefaults::Missing,
-                                input,
-                                self.loc_by_alias,
-                                &field.name
-                            ));
+                        }
+
+                        match field.validator.default_value(py, Some(field.name.as_str()), state) {
+                            Ok(Some(value)) => {
+                                // Default value exists, and passed validation if required
+                                control_flow!(output_dict.set_item(&field.name_py, value))?;
+                            },
+                            Ok(None) => {
+                                // This means there was no default value
+                                if (field.required) {
+                                    errors.push(field.lookup_key.error(
+                                        ErrorTypeDefaults::Missing,
+                                        input,
+                                        self.loc_by_alias,
+                                        &field.name
+                                    ));
+                                }
+                            },
+                            Err(ValError::Omit) => continue,
+                            Err(ValError::LineErrors(line_errors)) => {
+                                for err in line_errors {
+                                    // Note: this will always use the field name even if there is an alias
+                                    // However, we don't mind so much because this error can only happen if the
+                                    // default value fails validation, which is arguably a developer error.
+                                    // We could try to "fix" this in the future if desired.
+                                    errors.push(err);
+                                }
+                            }
+                            Err(err) => return ControlFlow::Break(err),
                         }
                     }
                     ControlFlow::Continue(())
@@ -232,46 +252,56 @@ impl Validator for TypedDictValidator {
                 if let Some(ref mut used_keys) = used_keys {
                     for item_result in <$iter>::new($dict)? {
                         let (raw_key, value) = item_result?;
-                        let either_str = match raw_key.strict_str() {
+                        let either_str = match raw_key.validate_str(true, false).map(ValidationMatch::into_inner) {
                             Ok(k) => k,
                             Err(ValError::LineErrors(line_errors)) => {
                                 for err in line_errors {
                                     errors.push(
                                         err.with_outer_location(raw_key.as_loc_item())
-                                            .with_type(ErrorTypeDefaults::InvalidKey),
+                                            .with_type(ErrorTypeDefaults::InvalidKey)
+                                            .into_owned(py)
                                     );
                                 }
                                 continue;
                             }
-                            Err(err) => return Err(err),
+                            Err(err) => return Err(err.into_owned(py)),
                         };
-                        if used_keys.contains(either_str.as_cow()?.as_ref()) {
+                        let cow = either_str.as_cow().map_err(|err| err.into_owned(py))?;
+                        if used_keys.contains(cow.as_ref()) {
                             continue;
                         }
 
+                        let value = value.borrow_input();
                         // Unknown / extra field
                         match self.extra_behavior {
                             ExtraBehavior::Forbid => {
-                                errors.push(ValLineError::new_with_loc(
-                                    ErrorTypeDefaults::ExtraForbidden,
-                                    value,
-                                    raw_key.as_loc_item(),
-                                ));
+                                errors.push(
+                                    ValLineError::new_with_loc(
+                                        ErrorTypeDefaults::ExtraForbidden,
+                                        value,
+                                        raw_key.as_loc_item(),
+                                    )
+                                    .into_owned(py)
+                                );
                             }
                             ExtraBehavior::Ignore => {}
                             ExtraBehavior::Allow => {
                             let py_key = either_str.as_py_string(py);
-                                if let Some(ref validator) = self.extra_validator {
+                                if let Some(ref validator) = self.extras_validator {
                                     match validator.validate(py, value, state) {
                                         Ok(value) => {
                                             output_dict.set_item(py_key, value)?;
                                         }
                                         Err(ValError::LineErrors(line_errors)) => {
                                             for err in line_errors {
-                                                errors.push(err.with_outer_location(raw_key.as_loc_item()));
+                                                errors.push(
+                                                    err
+                                                    .with_outer_location(raw_key.as_loc_item())
+                                                    .into_owned(py)
+                                                );
                                             }
                                         }
-                                        Err(err) => return Err(err),
+                                        Err(err) => return Err(err.into_owned(py)),
                                     }
                                 } else {
                                     output_dict.set_item(py_key, value.to_object(py))?;
@@ -284,8 +314,9 @@ impl Validator for TypedDictValidator {
         }
         match dict {
             GenericMapping::PyDict(d) => process!(d, py_get_dict_item, DictGenericIterator),
-            GenericMapping::PyGetAttr(d, kwargs) => process!(d, py_get_attr, AttributesGenericIterator, kwargs),
             GenericMapping::PyMapping(d) => process!(d, py_get_mapping_item, MappingGenericIterator),
+            GenericMapping::StringMapping(d) => process!(d, py_get_string_mapping_item, StringMappingGenericIterator),
+            GenericMapping::PyGetAttr(d, kwargs) => process!(d, py_get_attr, AttributesGenericIterator, kwargs),
             GenericMapping::JsonObject(d) => process!(d, json_get, JsonObjectGenericIterator),
         }
 
@@ -296,27 +327,7 @@ impl Validator for TypedDictValidator {
         }
     }
 
-    fn different_strict_behavior(
-        &self,
-        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        self.fields
-            .iter()
-            .any(|f| f.validator.different_strict_behavior(definitions, ultra_strict))
-    }
-
     fn get_name(&self) -> &str {
         Self::EXPECTED_TYPE
     }
-
-    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        self.fields
-            .iter_mut()
-            .try_for_each(|f| f.validator.complete(definitions))?;
-        match &mut self.extra_validator {
-            Some(v) => v.complete(definitions),
-            None => Ok(()),
-        }
-    }
 }
