@@ -8,7 +8,7 @@ use smallvec::SmallVec;
 
 use crate::build_tools::py_schema_err;
 use crate::build_tools::{is_strict, schema_or_config};
-use crate::errors::{ErrorType, LocItem, ValError, ValLineError, ValResult};
+use crate::errors::{AsLocItem, ErrorType, ValError, ValLineError, ValResult};
 use crate::input::{GenericMapping, Input};
 use crate::lookup_key::LookupKey;
 use crate::py_gc::PyGcTraverse;
@@ -16,40 +16,29 @@ use crate::tools::SchemaDict;
 
 use super::custom_error::CustomError;
 use super::literal::LiteralLookup;
-use super::{build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
+use super::{
+    build_validator, BuildValidator, CombinedValidator, DefinitionsBuilder, Exactness, ValidationState, Validator,
+};
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug)]
 enum UnionMode {
-    Smart {
-        strict_required: bool,
-        ultra_strict_required: bool,
-    },
+    Smart,
     LeftToRight,
 }
 
-impl UnionMode {
-    // construct smart with some default values
-    const fn default_smart() -> Self {
-        Self::Smart {
-            strict_required: true,
-            ultra_strict_required: false,
-        }
-    }
-}
-
 impl FromStr for UnionMode {
     type Err = PyErr;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         match s {
-            "smart" => Ok(Self::default_smart()),
+            "smart" => Ok(Self::Smart),
             "left_to_right" => Ok(Self::LeftToRight),
             s => py_schema_err!("Invalid union mode: `{}`, expected `smart` or `left_to_right`", s),
         }
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct UnionValidator {
     mode: UnionMode,
     choices: Vec<(CombinedValidator, Option<String>)>,
@@ -87,7 +76,7 @@ impl BuildValidator for UnionValidator {
         let auto_collapse = || schema.get_as_req(intern!(py, "auto_collapse")).unwrap_or(true);
         let mode = schema
             .get_as::<&str>(intern!(py, "mode"))?
-            .map_or(Ok(UnionMode::default_smart()), UnionMode::from_str)?;
+            .map_or(Ok(UnionMode::Smart), UnionMode::from_str)?;
         match choices.len() {
             0 => py_schema_err!("One or more union choices required"),
             1 if auto_collapse() => Ok(choices.into_iter().next().unwrap().0),
@@ -112,71 +101,74 @@ impl BuildValidator for UnionValidator {
 }
 
 impl UnionValidator {
-    fn validate_smart<'s, 'data>(
-        &'s self,
+    fn validate_smart<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
-        strict_required: bool,
-        ultra_strict_required: bool,
     ) -> ValResult<'data, PyObject> {
-        if ultra_strict_required {
-            // do an ultra strict check first
-            let state = &mut state.rebind_extra(|extra| {
-                extra.strict = Some(true);
-                extra.ultra_strict = true;
-            });
-            if let Some(res) = self
-                .choices
-                .iter()
-                .map(|(validator, _label)| validator.validate(py, input, state))
-                .find(ValResult::is_ok)
-            {
-                return res;
-            }
-        }
-
+        let old_exactness = state.exactness;
+        let strict = state.strict_or(self.strict);
         let mut errors = MaybeErrors::new(self.custom_error.as_ref());
 
-        if state.strict_or(self.strict) {
-            let state = &mut state.rebind_extra(|extra| extra.strict = Some(true));
-            for (validator, label) in &self.choices {
-                match validator.validate(py, input, state) {
-                    Err(ValError::LineErrors(lines)) => errors.push(validator, label.as_deref(), lines),
-                    otherwise => return otherwise,
-                };
-            }
+        let mut success = None;
 
-            Err(errors.into_val_error(input))
-        } else {
-            if strict_required {
-                // 1st pass: check if the value is an exact instance of one of the Union types,
-                // e.g. use validate in strict mode
-                let state = &mut state.rebind_extra(|extra| extra.strict = Some(true));
-                if let Some(res) = self
-                    .choices
-                    .iter()
-                    .map(|(validator, _label)| validator.validate(py, input, state))
-                    .find(ValResult::is_ok)
-                {
-                    return res;
+        for (choice, label) in &self.choices {
+            let state = &mut state.rebind_extra(|extra| {
+                if strict {
+                    extra.strict = Some(strict);
                 }
+            });
+            state.exactness = Some(Exactness::Exact);
+            let result = choice.validate(py, input, state);
+            match result {
+                Ok(new_success) => match state.exactness {
+                    // exact match, return
+                    Some(Exactness::Exact) => {
+                        return {
+                            // exact match, return, restore any previous exactness
+                            state.exactness = old_exactness;
+                            Ok(new_success)
+                        };
+                    }
+                    _ => {
+                        // success should always have an exactness
+                        debug_assert_ne!(state.exactness, None);
+                        let new_exactness = state.exactness.unwrap_or(Exactness::Lax);
+                        // if the new result has higher exactness than the current success, replace it
+                        if success
+                            .as_ref()
+                            .map_or(true, |(_, current_exactness)| *current_exactness < new_exactness)
+                        {
+                            // TODO: is there a possible optimization here, where once there has
+                            // been one success, we turn on strict mode, to avoid unnecessary
+                            // coercions for further validation?
+                            success = Some((new_success, new_exactness));
+                        }
+                    }
+                },
+                Err(ValError::LineErrors(lines)) => {
+                    // if we don't yet know this validation will succeed, record the error
+                    if success.is_none() {
+                        errors.push(choice, label.as_deref(), lines);
+                    }
+                }
+                otherwise => return otherwise,
             }
-
-            // 2nd pass: check if the value can be coerced into one of the Union types, e.g. use validate
-            for (validator, label) in &self.choices {
-                match validator.validate(py, input, state) {
-                    Err(ValError::LineErrors(lines)) => errors.push(validator, label.as_deref(), lines),
-                    otherwise => return otherwise,
-                };
-            }
-
-            Err(errors.into_val_error(input))
         }
+        state.exactness = old_exactness;
+
+        if let Some((success, exactness)) = success {
+            state.floor_exactness(exactness);
+            return Ok(success);
+        }
+
+        // no matches, build errors
+        Err(errors.into_val_error(input))
     }
 
-    fn validate_left_to_right<'s, 'data>(
-        &'s self,
+    fn validate_left_to_right<'data>(
+        &self,
         py: Python<'data>,
         input: &'data impl Input<'data>,
         state: &mut ValidationState,
@@ -217,47 +209,14 @@ impl Validator for UnionValidator {
         state: &mut ValidationState,
     ) -> ValResult<'data, PyObject> {
         match self.mode {
-            UnionMode::Smart {
-                strict_required,
-                ultra_strict_required,
-            } => self.validate_smart(py, input, state, strict_required, ultra_strict_required),
+            UnionMode::Smart => self.validate_smart(py, input, state),
             UnionMode::LeftToRight => self.validate_left_to_right(py, input, state),
         }
     }
 
-    fn different_strict_behavior(
-        &self,
-        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        self.choices
-            .iter()
-            .any(|(v, _)| v.different_strict_behavior(definitions, ultra_strict))
-    }
-
     fn get_name(&self) -> &str {
         &self.name
     }
-
-    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        self.choices.iter_mut().try_for_each(|(v, _)| v.complete(definitions))?;
-        if let UnionMode::Smart {
-            ref mut strict_required,
-            ref mut ultra_strict_required,
-        } = self.mode
-        {
-            *strict_required = self
-                .choices
-                .iter()
-                .any(|(v, _)| v.different_strict_behavior(Some(definitions), false));
-            *ultra_strict_required = self
-                .choices
-                .iter()
-                .any(|(v, _)| v.different_strict_behavior(Some(definitions), true));
-        }
-
-        Ok(())
-    }
 }
 
 struct ChoiceLineErrors<'a, 'data> {
@@ -357,7 +316,7 @@ impl PyGcTraverse for Discriminator {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct TaggedUnionValidator {
     discriminator: Discriminator,
     lookup: LiteralLookup<CombinedValidator>,
@@ -455,8 +414,9 @@ impl Validator for TaggedUnionValidator {
                 let dict = input.validate_model_fields(self.strict, from_attributes)?;
                 let tag = match dict {
                     GenericMapping::PyDict(dict) => find_validator!(py_get_dict_item, dict),
-                    GenericMapping::PyGetAttr(obj, kwargs) => find_validator!(py_get_attr, obj, kwargs),
                     GenericMapping::PyMapping(mapping) => find_validator!(py_get_mapping_item, mapping),
+                    GenericMapping::StringMapping(d) => find_validator!(py_get_dict_item, d),
+                    GenericMapping::PyGetAttr(obj, kwargs) => find_validator!(py_get_attr, obj, kwargs),
                     GenericMapping::JsonObject(mapping) => find_validator!(json_get, mapping),
                 }?;
                 self.find_call_validator(py, tag, input, state)
@@ -475,27 +435,9 @@ impl Validator for TaggedUnionValidator {
         }
     }
 
-    fn different_strict_behavior(
-        &self,
-        definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        self.lookup
-            .values
-            .iter()
-            .any(|v| v.different_strict_behavior(definitions, ultra_strict))
-    }
-
     fn get_name(&self) -> &str {
         &self.name
     }
-
-    fn complete(&mut self, definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        self.lookup
-            .values
-            .iter_mut()
-            .try_for_each(|validator| validator.complete(definitions))
-    }
 }
 
 impl TaggedUnionValidator {
@@ -506,8 +448,8 @@ impl TaggedUnionValidator {
     ) -> ValResult<'data, &'data PyString> {
         let dict = input.strict_dict()?;
         let either_tag = match dict {
-            GenericMapping::PyDict(dict) => match dict.get_item(intern!(py, "type")) {
-                Some(t) => t.strict_str()?,
+            GenericMapping::PyDict(dict) => match dict.get_item(intern!(py, "type"))? {
+                Some(t) => t.validate_str(true, false)?.into_inner(),
                 None => return Err(self.tag_not_found(input)),
             },
             _ => unreachable!(),
@@ -517,8 +459,8 @@ impl TaggedUnionValidator {
         // custom logic to distinguish between different function and tuple schemas
         if tag == "function" || tag == "tuple" {
             let mode = match dict {
-                GenericMapping::PyDict(dict) => match dict.get_item(intern!(py, "mode")) {
-                    Some(m) => Some(m.strict_str()?),
+                GenericMapping::PyDict(dict) => match dict.get_item(intern!(py, "mode"))? {
+                    Some(m) => Some(m.validate_str(true, false)?.into_inner()),
                     None => None,
                 },
                 _ => unreachable!(),
@@ -554,7 +496,7 @@ impl TaggedUnionValidator {
         if let Ok(Some((tag, validator))) = self.lookup.validate(py, tag) {
             return match validator.validate(py, input, state) {
                 Ok(res) => Ok(res),
-                Err(err) => Err(err.with_outer_location(LocItem::try_from(tag.to_object(py).into_ref(py))?)),
+                Err(err) => Err(err.with_outer_location(tag.as_loc_item())),
             };
         }
         match self.custom_error {
