@@ -16,6 +16,7 @@ use crate::tools::SchemaDict;
 use crate::url::{schema_is_special, PyMultiHostUrl, PyUrl};
 
 use super::literal::expected_repr_name;
+use super::Exactness;
 use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationState, Validator};
 
 type AllowedSchemas = Option<(AHashSet<String>, String)>;
@@ -87,32 +88,25 @@ impl Validator for UrlValidator {
             self.default_port,
             &self.default_path,
         ) {
-            Ok(()) => Ok(PyUrl::new(lib_url).into_py(py)),
+            Ok(()) => {
+                // Lax rather than strict to preserve V2.4 semantic that str wins over url in union
+                state.floor_exactness(Exactness::Lax);
+                Ok(PyUrl::new(lib_url).into_py(py))
+            }
             Err(error_type) => return Err(ValError::new(error_type, input)),
         }
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        !ultra_strict
-    }
-
     fn get_name(&self) -> &str {
         &self.name
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 impl UrlValidator {
     fn get_url<'s, 'data>(&'s self, input: &'data impl Input<'data>, strict: bool) -> ValResult<'data, Url> {
-        match input.validate_str(strict) {
-            Ok(either_str) => {
+        match input.validate_str(strict, false) {
+            Ok(val_match) => {
+                let either_str = val_match.into_inner();
                 let cow = either_str.as_cow()?;
                 let url_str = cow.as_ref();
 
@@ -227,32 +221,25 @@ impl Validator for MultiHostUrlValidator {
             self.default_port,
             &self.default_path,
         ) {
-            Ok(()) => Ok(multi_url.into_py(py)),
+            Ok(()) => {
+                // Lax rather than strict to preserve V2.4 semantic that str wins over url in union
+                state.floor_exactness(Exactness::Lax);
+                Ok(multi_url.into_py(py))
+            }
             Err(error_type) => return Err(ValError::new(error_type, input)),
         }
     }
 
-    fn different_strict_behavior(
-        &self,
-        _definitions: Option<&DefinitionsBuilder<CombinedValidator>>,
-        ultra_strict: bool,
-    ) -> bool {
-        !ultra_strict
-    }
-
     fn get_name(&self) -> &str {
         &self.name
     }
-
-    fn complete(&mut self, _definitions: &DefinitionsBuilder<CombinedValidator>) -> PyResult<()> {
-        Ok(())
-    }
 }
 
 impl MultiHostUrlValidator {
     fn get_url<'s, 'data>(&'s self, input: &'data impl Input<'data>, strict: bool) -> ValResult<'data, PyMultiHostUrl> {
-        match input.validate_str(strict) {
-            Ok(either_str) => {
+        match input.validate_str(strict, false) {
+            Ok(val_match) => {
+                let either_str = val_match.into_inner();
                 let cow = either_str.as_cow()?;
                 let url_str = cow.as_ref();
 
@@ -506,7 +493,7 @@ fn check_sub_defaults(
         if let Some(default_port) = default_port {
             lib_url
                 .set_port(Some(default_port))
-                .map_err(|_| map_parse_err(ParseError::EmptyHost))?;
+                .map_err(|()| map_parse_err(ParseError::EmptyHost))?;
         }
     }
     if let Some(ref default_path) = default_path {
