@@ -298,28 +298,28 @@ def test_definition_model_core(definition_model_data, benchmark):
         __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
     v = SchemaValidator(
-        {
-            'ref': 'Branch',
-            'type': 'model',
-            'cls': CoreBranch,
-            'schema': {
-                'type': 'model-fields',
-                'fields': {
-                    'width': {'type': 'model-field', 'schema': {'type': 'int'}},
-                    'branch': {
-                        'type': 'model-field',
-                        'schema': {
-                            'type': 'default',
-                            'schema': {
-                                'type': 'nullable',
-                                'schema': {'type': 'definition-ref', 'schema_ref': 'Branch'},
-                            },
-                            'default': None,
-                        },
-                    },
-                },
-            },
-        }
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema(schema_ref='Branch'),
+            [
+                core_schema.model_schema(
+                    CoreBranch,
+                    core_schema.model_fields_schema(
+                        {
+                            'width': core_schema.model_field(core_schema.int_schema()),
+                            'branch': core_schema.model_field(
+                                core_schema.with_default_schema(
+                                    core_schema.nullable_schema(
+                                        core_schema.definition_reference_schema(schema_ref='Branch')
+                                    ),
+                                    default=None,
+                                )
+                            ),
+                        }
+                    ),
+                    ref='Branch',
+                )
+            ],
+        )
     )
     benchmark(v.validate_python, definition_model_data)
 
@@ -791,7 +791,7 @@ def test_dont_raise_error(benchmark):
     def f(input_value, info):
         return input_value
 
-    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'general', 'function': f}})
+    v = SchemaValidator(core_schema.with_info_plain_validator_function(f))
 
     @benchmark
     def t():
@@ -803,7 +803,7 @@ def test_dont_raise_error_no_info(benchmark):
     def f(input_value):
         return input_value
 
-    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'no-info', 'function': f}})
+    v = SchemaValidator(core_schema.no_info_plain_validator_function(f))
 
     @benchmark
     def t():
@@ -815,7 +815,7 @@ def test_raise_error_value_error(benchmark):
     def f(input_value, info):
         raise ValueError('this is a custom error')
 
-    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'general', 'function': f}})
+    v = SchemaValidator(core_schema.with_info_plain_validator_function(f))
 
     @benchmark
     def t():
@@ -832,7 +832,7 @@ def test_raise_error_custom(benchmark):
     def f(input_value, info):
         raise PydanticCustomError('my_error', 'this is a custom error {foo}', {'foo': 'FOOBAR'})
 
-    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'general', 'function': f}})
+    v = SchemaValidator(core_schema.with_info_plain_validator_function(f))
 
     @benchmark
     def t():
@@ -875,7 +875,7 @@ def test_tuple_many_variable(benchmark):
 
 @pytest.mark.benchmark(group='tuple-many')
 def test_tuple_many_positional(benchmark):
-    v = SchemaValidator({'type': 'tuple-positional', 'items_schema': [], 'extra_schema': {'type': 'int'}})
+    v = SchemaValidator({'type': 'tuple-positional', 'items_schema': [], 'extras_schema': {'type': 'int'}})
     assert v.validate_python(list(range(10))) == tuple(range(10))
 
     benchmark(v.validate_python, list(range(10)))
@@ -927,10 +927,7 @@ def test_chain_list(benchmark):
     validator = SchemaValidator(
         {
             'type': 'chain',
-            'steps': [
-                {'type': 'str'},
-                {'type': 'function-plain', 'function': {'type': 'general', 'function': lambda v, info: Decimal(v)}},
-            ],
+            'steps': [{'type': 'str'}, core_schema.with_info_plain_validator_function(lambda v, info: Decimal(v))],
         }
     )
     assert validator.validate_python('42.42') == Decimal('42.42')
@@ -944,7 +941,7 @@ def test_chain_function(benchmark):
         {
             'type': 'function-after',
             'schema': {'type': 'str'},
-            'function': {'type': 'general', 'function': lambda v, info: Decimal(v)},
+            'function': {'type': 'with-info', 'function': lambda v, info: Decimal(v)},
         }
     )
     assert validator.validate_python('42.42') == Decimal('42.42')
@@ -959,8 +956,8 @@ def test_chain_two_functions(benchmark):
             'type': 'chain',
             'steps': [
                 {'type': 'str'},
-                {'type': 'function-plain', 'function': {'type': 'general', 'function': lambda v, info: Decimal(v)}},
-                {'type': 'function-plain', 'function': {'type': 'general', 'function': lambda v, info: v * 2}},
+                core_schema.with_info_plain_validator_function(lambda v, info: Decimal(v)),
+                core_schema.with_info_plain_validator_function(lambda v, info: v * 2),
             ],
         }
     )
@@ -977,9 +974,9 @@ def test_chain_nested_functions(benchmark):
             'schema': {
                 'type': 'function-after',
                 'schema': {'type': 'str'},
-                'function': {'type': 'general', 'function': lambda v, info: Decimal(v)},
+                'function': {'type': 'with-info', 'function': lambda v, info: Decimal(v)},
             },
-            'function': {'type': 'general', 'function': lambda v, info: v * 2},
+            'function': {'type': 'with-info', 'function': lambda v, info: v * 2},
         }
     )
     assert validator.validate_python('42.42') == Decimal('84.84')
@@ -1002,7 +999,7 @@ def generator_gen_python(v, validator, info):
 
 @pytest.mark.benchmark(group='generator')
 def test_generator_python(benchmark):
-    schema = core_schema.general_wrap_validator_function(generator_gen_python, {'type': 'int'})
+    schema = core_schema.with_info_wrap_validator_function(generator_gen_python, {'type': 'int'})
     v = SchemaValidator(schema)
     input_value = tuple(range(100))
 
@@ -1301,7 +1298,7 @@ def test_tagged_union_int_keys_json(benchmark):
 @skip_wasm_deep_stack
 @pytest.mark.benchmark(group='field_function_validator')
 def test_field_function_validator(benchmark) -> None:
-    def f(v: int, info: core_schema.FieldValidationInfo) -> int:
+    def f(v: int, info: core_schema.ValidationInfo) -> int:
         assert info.field_name == 'x'
         return v + 1
 
@@ -1309,7 +1306,7 @@ def test_field_function_validator(benchmark) -> None:
     limit = pydantic_core._pydantic_core._recursion_limit - 3
 
     for _ in range(limit):
-        schema = core_schema.field_after_validator_function(f, 'x', schema)
+        schema = core_schema.with_info_after_validator_function(f, schema, field_name='x')
 
     schema = core_schema.typed_dict_schema({'x': core_schema.typed_dict_field(schema)})
 
