@@ -1,5 +1,6 @@
 import dataclasses
 import json
+import platform
 import sys
 from collections import namedtuple
 from datetime import date, datetime, time, timedelta, timezone
@@ -11,12 +12,18 @@ from typing import ClassVar
 import pytest
 from dirty_equals import HasRepr, IsList
 
+import pydantic_core
 from pydantic_core import PydanticSerializationError, SchemaSerializer, SchemaValidator, core_schema, to_json
 
 from ..conftest import plain_repr
 from .test_dataclasses import IsStrictDict, on_pypy
 from .test_list_tuple import as_list, as_tuple
 
+try:
+    import numpy
+except ImportError:
+    numpy = None
+
 
 @pytest.fixture(scope='module')
 def any_serializer():
@@ -299,10 +306,10 @@ def test_unknown_type(any_serializer: SchemaSerializer):
     f = Foobar()
     assert any_serializer.to_python(f) == f
 
-    with pytest.raises(PydanticSerializationError, match='Unable to serialize unknown type: <Foobar repr>'):
+    with pytest.raises(PydanticSerializationError, match="Unable to serialize unknown type: <class '.+Foobar'>"):
         any_serializer.to_python(f, mode='json')
 
-    with pytest.raises(PydanticSerializationError, match='Unable to serialize unknown type: <Foobar repr>'):
+    with pytest.raises(PydanticSerializationError, match="Unable to serialize unknown type: <class '.+Foobar'>"):
         any_serializer.to_json(f)
 
 
@@ -343,6 +350,10 @@ class FoobarCount:
         return f'<FoobarCount {self.v} repr>'
 
 
+@pytest.mark.skipif(
+    platform.python_implementation() == 'PyPy' and pydantic_core._pydantic_core.build_profile == 'debug',
+    reason='PyPy does not have enough stack space for Rust debug builds to recurse very deep',
+)
 def test_fallback_cycle_change(any_serializer: SchemaSerializer):
     v = 1
 
@@ -359,8 +370,9 @@ def test_fallback_cycle_change(any_serializer: SchemaSerializer):
 
     f = FoobarCount(0)
     v = 0
-    # because when recursion is detected and we're in mode python, we just return the value
-    assert any_serializer.to_python(f, fallback=fallback_func) == HasRepr('<FoobarCount 201 repr>')
+    # when recursion is detected and we're in mode python, we just return the value
+    expected_visits = pydantic_core._pydantic_core._recursion_limit - 1
+    assert any_serializer.to_python(f, fallback=fallback_func) == HasRepr(f'<FoobarCount {expected_visits} repr>')
 
     with pytest.raises(ValueError, match=r'Circular reference detected \(depth exceeded\)'):
         any_serializer.to_json(f, fallback=fallback_func)
@@ -493,6 +505,14 @@ def test_any_model():
     assert s.to_python(Foo(a='hello', b=b'more'), exclude={'a'}) == IsStrictDict()
     assert s.to_json(Foo(a='hello', b=b'more'), exclude={'a'}) == b'{}'
 
+    assert s.to_python(Foo) == Foo
+    with pytest.raises(PydanticSerializationError, match=r"Unable to serialize unknown type: <class 'type'>"):
+        s.to_python(Foo, mode='json')
+    with pytest.raises(PydanticSerializationError, match=r"Unable to serialize unknown type: <class 'type'>"):
+        s.to_json(Foo)
+    assert s.to_python(Foo, mode='json', fallback=lambda x: x.__name__) == 'Foo'
+    assert s.to_json(Foo, fallback=lambda x: x.__name__) == b'"Foo"'
+
 
 def test_dataclass_classvar(any_serializer):
     @dataclasses.dataclass
@@ -567,3 +587,18 @@ def test_slots_mixed(any_serializer):
 
     assert any_serializer.to_python(dc) == {'x': 1, 'x2': 2}
     assert any_serializer.to_json(dc) == b'{"x":1,"x2":2}'
+
+
+@pytest.mark.skipif(numpy is None, reason='numpy is not installed')
+def test_numpy_float(any_serializer):
+    assert any_serializer.to_python(numpy.float64(1.0)) == 1.0
+    assert any_serializer.to_python(numpy.float64(1.0), mode='json') == 1.0
+    assert any_serializer.to_json(numpy.float64(1.0)) == b'1.0'
+
+    # float16 is not a subclass of float
+    assert not isinstance(numpy.float16(1.0), float)
+    assert any_serializer.to_python(numpy.float16(1.0)) == 1.0
+    with pytest.raises(PydanticSerializationError, match=r"Unable to serialize unknown type: <class 'numpy\.float16'>"):
+        any_serializer.to_python(numpy.float16(1.0), mode='json')
+    with pytest.raises(PydanticSerializationError, match=r"Unable to serialize unknown type: <class 'numpy\.float16'>"):
+        any_serializer.to_json(numpy.float16(1.0))
