@@ -2,7 +2,7 @@ import dataclasses
 import json
 import platform
 from random import randint
-from typing import Any, ClassVar
+from typing import Any, ClassVar, Dict
 
 try:
     from functools import cached_property
@@ -570,6 +570,9 @@ def test_property():
     assert s.to_python(Model(width=3, height=4), mode='json') == {'width': 3, 'height': 4, 'area': '12'}
     assert s.to_json(Model(width=3, height=4)) == b'{"width":3,"height":4,"area":"12"}'
 
+    assert s.to_python(Model(width=3, height=4), round_trip=True) == {'width': 3, 'height': 4}
+    assert s.to_json(Model(width=3, height=4), round_trip=True) == b'{"width":3,"height":4}'
+
 
 def test_property_alias():
     @dataclasses.dataclass
@@ -915,3 +918,27 @@ def test_extra_config_nested_model():
     s_repr = plain_repr(s)
     assert 'has_extra:true,root_model:false,name:"InnerModel"' in s_repr
     assert 'has_extra:false,root_model:false,name:"OuterModel"' in s_repr
+
+
+def test_extra_custom_serializer():
+    class Model:
+        __slots__ = ('__pydantic_extra__', '__dict__')
+        __pydantic_extra__: Dict[str, Any]
+
+    schema = core_schema.model_schema(
+        Model,
+        core_schema.model_fields_schema(
+            {},
+            extra_behavior='allow',
+            extras_schema=core_schema.any_schema(
+                serialization=core_schema.plain_serializer_function_ser_schema(lambda v: v + ' bam!')
+            ),
+        ),
+        extra_behavior='allow',
+    )
+    s = SchemaSerializer(schema)
+
+    m = Model()
+    m.__pydantic_extra__ = {'extra': 'extra'}
+
+    assert s.to_python(m) == {'extra': 'extra bam!'}
