@@ -1,11 +1,15 @@
+import enum
 import re
+import sys
 from decimal import Decimal
 from typing import Any, Optional
+from unittest.mock import patch
 
 import pytest
 from dirty_equals import HasRepr, IsInstance, IsJson, IsStr
 
 from pydantic_core import (
+    CoreConfig,
     PydanticCustomError,
     PydanticKnownError,
     PydanticOmit,
@@ -60,7 +64,7 @@ def test_pydantic_value_error_usage():
     def f(input_value, info):
         raise PydanticCustomError('my_error', 'this is a custom error {foo} {bar}', {'foo': 'FOOBAR', 'bar': 42})
 
-    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'general', 'function': f}})
+    v = SchemaValidator(core_schema.with_info_plain_validator_function(f))
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(42)
@@ -80,7 +84,7 @@ def test_pydantic_value_error_invalid_dict():
     def my_function(input_value, info):
         raise PydanticCustomError('my_error', 'this is a custom error {foo}', {(): 'foobar'})
 
-    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'general', 'function': my_function}})
+    v = SchemaValidator(core_schema.with_info_plain_validator_function(my_function))
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(42)
@@ -98,7 +102,7 @@ def test_pydantic_value_error_invalid_type():
     def f(input_value, info):
         raise PydanticCustomError('my_error', 'this is a custom error {foo}', [('foo', 123)])
 
-    v = SchemaValidator({'type': 'function-plain', 'function': {'type': 'general', 'function': f}})
+    v = SchemaValidator(core_schema.with_info_plain_validator_function(f))
 
     with pytest.raises(TypeError, match="argument 'context': 'list' object cannot be converted to 'PyDict'"):
         v.validate_python(42)
@@ -114,9 +118,7 @@ def test_validator_instance_plain():
             return f'{input_value} {self.foo} {self.bar}'
 
     c = CustomValidator()
-    v = SchemaValidator(
-        {'type': 'function-plain', 'metadata': {'instance': c}, 'function': {'type': 'general', 'function': c.validate}}
-    )
+    v = SchemaValidator(core_schema.with_info_plain_validator_function(c.validate, metadata={'instance': c}))
     c.foo += 1
 
     assert v.validate_python('input value') == 'input value 43 before'
@@ -135,12 +137,7 @@ def test_validator_instance_after():
 
     c = CustomValidator()
     v = SchemaValidator(
-        {
-            'type': 'function-after',
-            'metadata': {'instance': c},
-            'function': {'type': 'general', 'function': c.validate},
-            'schema': {'type': 'str'},
-        }
+        core_schema.with_info_after_validator_function(c.validate, core_schema.str_schema(), metadata={'instance': c})
     )
     c.foo += 1
 
@@ -171,9 +168,7 @@ def test_pydantic_error_type_raise_no_ctx():
     def f(input_value, info):
         raise PydanticKnownError('finite_number')
 
-    v = SchemaValidator(
-        {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
-    )
+    v = SchemaValidator(core_schema.with_info_before_validator_function(f, core_schema.int_schema()))
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(4)
@@ -192,9 +187,7 @@ def test_pydantic_error_type_raise_ctx(extra: dict):
     def f(input_value, info):
         raise PydanticKnownError('greater_than', ctx)
 
-    v = SchemaValidator(
-        {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
-    )
+    v = SchemaValidator(core_schema.with_info_before_validator_function(f, core_schema.int_schema()))
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(4)
@@ -209,9 +202,7 @@ def test_pydantic_error_type_raise_custom_no_ctx(ctx: Optional[dict]):
     def f(input_value, info):
         raise PydanticKnownError('int_type', ctx)
 
-    v = SchemaValidator(
-        {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
-    )
+    v = SchemaValidator(core_schema.with_info_before_validator_function(f, core_schema.int_schema()))
 
     expect_ctx = {'ctx': {}} if ctx is not None else {}
 
@@ -232,9 +223,7 @@ def test_pydantic_custom_error_type_raise_custom_ctx(extra: dict):
     def f(input_value, info):
         raise PydanticCustomError('my_error', 'my message with {val}', ctx)
 
-    v = SchemaValidator(
-        {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
-    )
+    v = SchemaValidator(core_schema.with_info_before_validator_function(f, core_schema.int_schema()))
 
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python(4)
@@ -249,9 +238,7 @@ def test_pydantic_custom_error_type_raise_custom_no_ctx(ctx: Optional[dict]):
     def f(input_value, info):
         raise PydanticCustomError('my_error', 'my message', ctx)
 
-    v = SchemaValidator(
-        {'type': 'function-before', 'function': {'type': 'general', 'function': f}, 'schema': {'type': 'int'}}
-    )
+    v = SchemaValidator(core_schema.with_info_before_validator_function(f, core_schema.int_schema()))
 
     expect_ctx = {'ctx': {}} if ctx is not None else {}
 
@@ -302,7 +289,9 @@ all_errors = [
     ('string_unicode', 'Input should be a valid string, unable to parse raw data as a unicode string', None),
     ('string_pattern_mismatch', "String should match pattern 'foo'", {'pattern': 'foo'}),
     ('string_too_short', 'String should have at least 42 characters', {'min_length': 42}),
+    ('string_too_short', 'String should have at least 1 character', {'min_length': 1}),
     ('string_too_long', 'String should have at most 42 characters', {'max_length': 42}),
+    ('string_too_long', 'String should have at most 1 character', {'max_length': 1}),
     ('dict_type', 'Input should be a valid dictionary', None),
     ('mapping_type', 'Input should be a valid mapping, error: foobar', {'error': 'foobar'}),
     ('iterable_type', 'Input should be iterable', None),
@@ -325,7 +314,9 @@ all_errors = [
     ('float_parsing', 'Input should be a valid number, unable to parse string as a number', None),
     ('bytes_type', 'Input should be a valid bytes', None),
     ('bytes_too_short', 'Data should have at least 42 bytes', {'min_length': 42}),
+    ('bytes_too_short', 'Data should have at least 1 byte', {'min_length': 1}),
     ('bytes_too_long', 'Data should have at most 42 bytes', {'max_length': 42}),
+    ('bytes_too_long', 'Data should have at most 1 byte', {'max_length': 1}),
     ('value_error', 'Value error, foobar', {'error': ValueError('foobar')}),
     ('assertion_error', 'Assertion failed, foobar', {'error': AssertionError('foobar')}),
     ('literal_error', 'Input should be foo', {'expected': 'foo'}),
@@ -369,6 +360,7 @@ all_errors = [
     ('url_parsing', 'Input should be a valid URL, Foobar', {'error': 'Foobar'}),
     ('url_syntax_violation', 'Input violated strict URL syntax rules, Foobar', {'error': 'Foobar'}),
     ('url_too_long', 'URL should have at most 42 characters', {'max_length': 42}),
+    ('url_too_long', 'URL should have at most 1 character', {'max_length': 1}),
     ('url_scheme', 'URL scheme should be "foo", "bar" or "spam"', {'expected_schemes': '"foo", "bar" or "spam"'}),
     ('uuid_type', 'UUID input should be a string, bytes or UUID object', None),
     ('uuid_parsing', 'Input should be a valid UUID, Foobar', {'error': 'Foobar'}),
@@ -376,12 +368,19 @@ all_errors = [
     ('decimal_type', 'Decimal input should be an integer, float, string or Decimal object', None),
     ('decimal_parsing', 'Input should be a valid decimal', None),
     ('decimal_max_digits', 'Decimal input should have no more than 42 digits in total', {'max_digits': 42}),
+    ('decimal_max_digits', 'Decimal input should have no more than 1 digit in total', {'max_digits': 1}),
     ('decimal_max_places', 'Decimal input should have no more than 42 decimal places', {'decimal_places': 42}),
+    ('decimal_max_places', 'Decimal input should have no more than 1 decimal place', {'decimal_places': 1}),
     (
         'decimal_whole_digits',
         'Decimal input should have no more than 42 digits before the decimal point',
         {'whole_digits': 42},
     ),
+    (
+        'decimal_whole_digits',
+        'Decimal input should have no more than 1 digit before the decimal point',
+        {'whole_digits': 1},
+    ),
 ]
 
 
@@ -418,28 +417,19 @@ def test_pydantic_value_error_plain(py_and_json: PyAndJson):
     def f(input_value, info):
         raise PydanticCustomError
 
-    v = py_and_json({'type': 'function-plain', 'function': {'type': 'general', 'function': f}})
+    v = py_and_json(core_schema.with_info_plain_validator_function(f))
     with pytest.raises(TypeError, match='missing 2 required positional arguments'):
         v.validate_test('4')
 
 
 @pytest.mark.parametrize('exception', [PydanticOmit(), PydanticOmit])
 def test_list_omit_exception(py_and_json: PyAndJson, exception):
-    def f(input_value, info):
+    def f(input_value):
         if input_value % 2 == 0:
             raise exception
         return input_value
 
-    v = py_and_json(
-        {
-            'type': 'list',
-            'items_schema': {
-                'type': 'function-after',
-                'schema': {'type': 'int'},
-                'function': {'type': 'general', 'function': f},
-            },
-        }
-    )
+    v = py_and_json(core_schema.list_schema(core_schema.no_info_after_validator_function(f, core_schema.int_schema())))
     assert v.validate_test([1, 2, '3', '4']) == [1, 3]
 
 
@@ -517,6 +507,212 @@ def test_all_errors():
         pytest.fail('core_schema.ErrorType needs to be updated')
 
 
+@pytest.mark.skipif(sys.version_info < (3, 11), reason='This is the modern version used post 3.10.')
+def test_validation_error_cause_contents():
+    enabled_config: CoreConfig = {'validation_error_cause': True}
+
+    def multi_raise_py_error(v: Any) -> Any:
+        try:
+            raise AssertionError('Wrong')
+        except AssertionError as e:
+            raise ValueError('Oh no!') from e
+
+    s2 = SchemaValidator(core_schema.no_info_plain_validator_function(multi_raise_py_error), config=enabled_config)
+    with pytest.raises(ValidationError) as exc_info:
+        s2.validate_python('anything')
+
+    cause_group = exc_info.value.__cause__
+    assert isinstance(cause_group, BaseExceptionGroup)
+    assert len(cause_group.exceptions) == 1
+
+    cause = cause_group.exceptions[0]
+    assert cause.__notes__
+    assert cause.__notes__[-1].startswith('\nPydantic: ')
+    assert repr(cause) == repr(ValueError('Oh no!'))
+    assert cause.__traceback__ is not None
+
+    sub_cause = cause.__cause__
+    assert repr(sub_cause) == repr(AssertionError('Wrong'))
+    assert sub_cause.__cause__ is None
+    assert sub_cause.__traceback__ is not None
+
+    # Edge case: make sure a deep inner ValidationError(s) causing a validator failure doesn't cause any problems:
+    def outer_raise_py_error(v: Any) -> Any:
+        try:
+            s2.validate_python('anything')
+        except ValidationError as e:
+            raise ValueError('Sub val failure') from e
+
+    s3 = SchemaValidator(core_schema.no_info_plain_validator_function(outer_raise_py_error), config=enabled_config)
+    with pytest.raises(ValidationError) as exc_info:
+        s3.validate_python('anything')
+
+    assert isinstance(exc_info.value.__cause__, BaseExceptionGroup)
+    assert len(exc_info.value.__cause__.exceptions) == 1
+    cause = exc_info.value.__cause__.exceptions[0]
+    assert cause.__notes__ and cause.__notes__[-1].startswith('\nPydantic: ')
+    assert repr(cause) == repr(ValueError('Sub val failure'))
+    subcause = cause.__cause__
+    assert isinstance(subcause, ValidationError)
+
+    cause_group = subcause.__cause__
+    assert isinstance(cause_group, BaseExceptionGroup)
+    assert len(cause_group.exceptions) == 1
+
+    cause = cause_group.exceptions[0]
+    assert cause.__notes__
+    assert cause.__notes__[-1].startswith('\nPydantic: ')
+    assert repr(cause) == repr(ValueError('Oh no!'))
+    assert cause.__traceback__ is not None
+
+    sub_cause = cause.__cause__
+    assert repr(sub_cause) == repr(AssertionError('Wrong'))
+    assert sub_cause.__cause__ is None
+    assert sub_cause.__traceback__ is not None
+
+
+@pytest.mark.skipif(sys.version_info >= (3, 11), reason='This is the backport/legacy version used pre 3.11 only.')
+def test_validation_error_cause_contents_legacy():
+    from exceptiongroup import BaseExceptionGroup
+
+    enabled_config: CoreConfig = {'validation_error_cause': True}
+
+    def multi_raise_py_error(v: Any) -> Any:
+        try:
+            raise AssertionError('Wrong')
+        except AssertionError as e:
+            raise ValueError('Oh no!') from e
+
+    s2 = SchemaValidator(core_schema.no_info_plain_validator_function(multi_raise_py_error), config=enabled_config)
+    with pytest.raises(ValidationError) as exc_info:
+        s2.validate_python('anything')
+
+    cause_group = exc_info.value.__cause__
+    assert isinstance(cause_group, BaseExceptionGroup)
+    assert len(cause_group.exceptions) == 1
+
+    cause = cause_group.exceptions[0]
+    assert repr(cause).startswith("UserWarning('Pydantic: ")
+
+    assert cause.__cause__ is not None
+    cause = cause.__cause__
+    assert repr(cause) == repr(ValueError('Oh no!'))
+    assert cause.__traceback__ is not None
+
+    sub_cause = cause.__cause__
+    assert repr(sub_cause) == repr(AssertionError('Wrong'))
+    assert sub_cause.__cause__ is None
+    assert sub_cause.__traceback__ is not None
+
+    # Make sure a deep inner ValidationError(s) causing a validator failure doesn't cause any problems:
+    def outer_raise_py_error(v: Any) -> Any:
+        try:
+            s2.validate_python('anything')
+        except ValidationError as e:
+            raise ValueError('Sub val failure') from e
+
+    s3 = SchemaValidator(core_schema.no_info_plain_validator_function(outer_raise_py_error), config=enabled_config)
+    with pytest.raises(ValidationError) as exc_info:
+        s3.validate_python('anything')
+
+    assert isinstance(exc_info.value.__cause__, BaseExceptionGroup)
+    assert len(exc_info.value.__cause__.exceptions) == 1
+    cause = exc_info.value.__cause__.exceptions[0]
+    assert repr(cause).startswith("UserWarning('Pydantic: ")
+    assert cause.__cause__ is not None
+    cause = cause.__cause__
+    assert repr(cause) == repr(ValueError('Sub val failure'))
+    subcause = cause.__cause__
+    assert isinstance(subcause, ValidationError)
+
+    cause_group = subcause.__cause__
+    assert isinstance(cause_group, BaseExceptionGroup)
+    assert len(cause_group.exceptions) == 1
+
+    cause = cause_group.exceptions[0]
+    assert repr(cause).startswith("UserWarning('Pydantic: ")
+    assert cause.__cause__ is not None
+    cause = cause.__cause__
+    assert repr(cause) == repr(ValueError('Oh no!'))
+    assert cause.__traceback__ is not None
+
+    sub_cause = cause.__cause__
+    assert repr(sub_cause) == repr(AssertionError('Wrong'))
+    assert sub_cause.__cause__ is None
+    assert sub_cause.__traceback__ is not None
+
+
+class CauseResult(enum.Enum):
+    CAUSE = enum.auto()
+    NO_CAUSE = enum.auto()
+    IMPORT_ERROR = enum.auto()
+
+
+@pytest.mark.parametrize(
+    'desc,config,expected_result',
+    [  # Without the backport should still work after 3.10 as not needed:
+        (
+            'Enabled',
+            {'validation_error_cause': True},
+            CauseResult.CAUSE if sys.version_info >= (3, 11) else CauseResult.IMPORT_ERROR,
+        ),
+        ('Disabled specifically', {'validation_error_cause': False}, CauseResult.NO_CAUSE),
+        ('Disabled implicitly', {}, CauseResult.NO_CAUSE),
+    ],
+)
+def test_validation_error_cause_config_variants(desc: str, config: CoreConfig, expected_result: CauseResult):
+    # Simulate the package being missing:
+    with patch.dict('sys.modules', {'exceptiongroup': None}):
+
+        def singular_raise_py_error(v: Any) -> Any:
+            raise ValueError('Oh no!')
+
+        s = SchemaValidator(core_schema.no_info_plain_validator_function(singular_raise_py_error), config=config)
+
+        if expected_result is CauseResult.IMPORT_ERROR:
+            # Confirm error message contains "requires the exceptiongroup module" in the middle of the string:
+            with pytest.raises(ImportError, match='requires the exceptiongroup module'):
+                s.validate_python('anything')
+        elif expected_result is CauseResult.CAUSE:
+            with pytest.raises(ValidationError) as exc_info:
+                s.validate_python('anything')
+            assert exc_info.value.__cause__ is not None
+            assert hasattr(exc_info.value.__cause__, 'exceptions')
+            assert len(exc_info.value.__cause__.exceptions) == 1
+            assert repr(exc_info.value.__cause__.exceptions[0]) == repr(ValueError('Oh no!'))
+        elif expected_result is CauseResult.NO_CAUSE:
+            with pytest.raises(ValidationError) as exc_info:
+                s.validate_python('anything')
+            assert exc_info.value.__cause__ is None
+        else:
+            raise AssertionError('Unhandled result: {}'.format(expected_result))
+
+
+def test_validation_error_cause_traceback_preserved():
+    """Makes sure historic bug of traceback being lost is fixed."""
+
+    enabled_config: CoreConfig = {'validation_error_cause': True}
+
+    def singular_raise_py_error(v: Any) -> Any:
+        raise ValueError('Oh no!')
+
+    s1 = SchemaValidator(core_schema.no_info_plain_validator_function(singular_raise_py_error), config=enabled_config)
+    with pytest.raises(ValidationError) as exc_info:
+        s1.validate_python('anything')
+
+    base_errs = getattr(exc_info.value.__cause__, 'exceptions', [])
+    assert len(base_errs) == 1
+    base_err = base_errs[0]
+
+    # Get to the root error:
+    cause = base_err
+    while cause.__cause__ is not None:
+        cause = cause.__cause__
+
+    # Should still have a traceback:
+    assert cause.__traceback__ is not None
+
+
 class BadRepr:
     def __repr__(self):
         raise RuntimeError('bad repr')
@@ -593,7 +789,7 @@ def test_error_json_python_error(pydantic_version: str):
     with pytest.raises(ValidationError) as exc_info:
         s.validate_python('anything')
 
-    exc = exc_info.value.errors()[0]['ctx']['error']  # type: ignore
+    exc = exc_info.value.errors()[0]['ctx']['error']
     assert isinstance(exc, ValueError)
     assert isinstance(exc.__context__, AssertionError)
 
@@ -854,9 +1050,27 @@ def test_loc_with_dots(pydantic_version):
     ]
     # insert_assert(str(exc_info.value))
     assert str(exc_info.value) == (
-        "1 validation error for typed-dict\n"
-        "`foo.bar`.0\n"
-        "  Input should be a valid integer, unable to parse string as an integer "
+        '1 validation error for typed-dict\n'
+        '`foo.bar`.0\n'
+        '  Input should be a valid integer, unable to parse string as an integer '
         "[type=int_parsing, input_value='x', input_type=str]\n"
         f'    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/int_parsing'
     )
+
+
+def test_hide_input_in_error() -> None:
+    s = SchemaValidator({'type': 'int'})
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python('definitely not an int')
+
+    for error in exc_info.value.errors(include_input=False):
+        assert 'input' not in error
+
+
+def test_hide_input_in_json() -> None:
+    s = SchemaValidator({'type': 'int'})
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python('definitely not an int')
+
+    for error in exc_info.value.errors(include_input=False):
+        assert 'input' not in error
