@@ -4,7 +4,7 @@ import re
 from typing import List
 
 import pytest
-from dirty_equals import IsList
+from dirty_equals import IsFloatNan, IsList
 
 import pydantic_core
 from pydantic_core import (
@@ -142,7 +142,7 @@ def test_error_loc():
             'fields': {
                 'field_a': {'type': 'typed-dict-field', 'schema': {'type': 'list', 'items_schema': {'type': 'int'}}}
             },
-            'extra_validator': {'type': 'int'},
+            'extras_schema': {'type': 'int'},
             'extra_behavior': 'allow',
         }
     )
@@ -254,18 +254,23 @@ def test_to_jsonable_python_schema_serializer():
 
     # force a recursive model to ensure we exercise the transfer of definitions from the loaded
     # serializer
-    c = core_schema.model_schema(
-        Foobar,
-        core_schema.typed_dict_schema(
-            {
-                'my_foo': core_schema.typed_dict_field(core_schema.int_schema(), serialization_alias='myFoo'),
-                'my_inners': core_schema.typed_dict_field(
-                    core_schema.list_schema(core_schema.definition_reference_schema('foobar')),
-                    serialization_alias='myInners',
+    c = core_schema.definitions_schema(
+        core_schema.definition_reference_schema(schema_ref='foobar'),
+        [
+            core_schema.model_schema(
+                Foobar,
+                core_schema.typed_dict_schema(
+                    {
+                        'my_foo': core_schema.typed_dict_field(core_schema.int_schema(), serialization_alias='myFoo'),
+                        'my_inners': core_schema.typed_dict_field(
+                            core_schema.list_schema(core_schema.definition_reference_schema('foobar')),
+                            serialization_alias='myInners',
+                        ),
+                    }
                 ),
-            }
-        ),
-        ref='foobar',
+                ref='foobar',
+            )
+        ],
     )
     v = SchemaValidator(c)
     s = SchemaSerializer(c)
@@ -353,3 +358,10 @@ def test_bad_repr():
         to_json(b)
 
     assert to_json(b, serialize_unknown=True) == b'"<Unserializable BadRepr object>"'
+
+
+def test_inf_nan_allow():
+    v = SchemaValidator(core_schema.float_schema(allow_inf_nan=True))
+    assert v.validate_json('Infinity') == float('inf')
+    assert v.validate_json('-Infinity') == float('-inf')
+    assert v.validate_json('NaN') == IsFloatNan()
