@@ -621,7 +621,7 @@ def test_internal_error(py_and_json: PyAndJson):
                 {
                     'name': 'b',
                     'mode': 'positional_only',
-                    'schema': {'type': 'function-plain', 'function': {'type': 'general', 'function': double_or_bust}},
+                    'schema': core_schema.with_info_plain_validator_function(double_or_bust),
                 },
             ],
         }
@@ -1009,11 +1009,11 @@ def test_error_display(pydantic_version):
     ]
     # insert_assert(str(exc_info.value))
     assert str(exc_info.value) == (
-        "1 validation error for arguments\n"
-        "b\n"
-        "  Missing required argument [type=missing_argument, "
+        '1 validation error for arguments\n'
+        'b\n'
+        '  Missing required argument [type=missing_argument, '
         "input_value=ArgsKwargs((), {'a': 1}), input_type=ArgsKwargs]\n"
-        f"    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/missing_argument"
+        f'    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/missing_argument'
     )
     # insert_assert(exc_info.value.json(include_url=False))
     assert exc_info.value.json(include_url=False) == (
