@@ -1,6 +1,5 @@
 import copy
 import json
-import pickle
 import platform
 import re
 from datetime import date, datetime, time, timedelta, timezone, tzinfo
@@ -10,7 +9,7 @@ from typing import Dict
 import pytest
 import pytz
 
-from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema
+from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema, validate_core_schema
 
 from ..conftest import Err, PyAndJson
 
@@ -37,6 +36,8 @@ from ..conftest import Err, PyAndJson
         (float('nan'), Err('Input should be a valid datetime, NaN values not permitted [type=datetime_parsing,')),
         (float('inf'), Err('Input should be a valid datetime, dates after 9999')),
         (float('-inf'), Err('Input should be a valid datetime, dates before 1600')),
+        ('-', Err('Input should be a valid datetime, input is too short [type=datetime_parsing,')),
+        ('+', Err('Input should be a valid datetime, input is too short [type=datetime_parsing,')),
     ],
 )
 def test_datetime(input_value, expected):
@@ -284,7 +285,7 @@ def test_union():
 
 def test_invalid_constraint():
     with pytest.raises(SchemaError, match=r'datetime\.gt\n  Input should be a valid datetime'):
-        SchemaValidator({'type': 'datetime', 'gt': 'foobar'})
+        validate_core_schema({'type': 'datetime', 'gt': 'foobar'})
 
 
 @pytest.mark.parametrize(
@@ -387,7 +388,7 @@ def test_mock_utc_offset_8_hours(mocker):
 
 def test_offset_too_large():
     with pytest.raises(SchemaError, match=r'Input should be greater than -86400 \[type=greater_than,'):
-        SchemaValidator(core_schema.datetime_schema(now_op='past', now_utc_offset=-24 * 3600))
+        validate_core_schema(core_schema.datetime_schema(now_op='past', now_utc_offset=-24 * 3600))
 
 
 def test_raises_schema_error_for_unknown_constraint_kind():
@@ -395,7 +396,7 @@ def test_raises_schema_error_for_unknown_constraint_kind():
         SchemaError,
         match=(r'Input should be \'aware\' or \'naive\' \[type=literal_error, input_value=\'foo\', input_type=str\]'),
     ):
-        SchemaValidator({'type': 'datetime', 'tz_constraint': 'foo'})
+        validate_core_schema({'type': 'datetime', 'tz_constraint': 'foo'})
 
 
 def test_aware():
@@ -477,18 +478,7 @@ def test_tz_constraint_too_high():
 
 def test_tz_constraint_wrong():
     with pytest.raises(SchemaError, match="Input should be 'aware' or 'naive"):
-        SchemaValidator(core_schema.datetime_schema(tz_constraint='wrong'))
-
-
-def test_tz_pickle() -> None:
-    """
-    https://github.com/pydantic/pydantic-core/issues/589
-    """
-    v = SchemaValidator(core_schema.datetime_schema())
-    original = datetime(2022, 6, 8, 12, 13, 14, tzinfo=timezone(timedelta(hours=-12, minutes=-15)))
-    validated = v.validate_python('2022-06-08T12:13:14-12:15')
-    assert validated == original
-    assert pickle.loads(pickle.dumps(validated)) == validated == original
+        validate_core_schema(core_schema.datetime_schema(tz_constraint='wrong'))
 
 
 def test_tz_hash() -> None:
