@@ -1,3 +1,4 @@
+import datetime
 import platform
 from dataclasses import dataclass
 from typing import List, Optional
@@ -13,39 +14,6 @@ from .test_typed_dict import Cls
 
 
 def test_branch_nullable():
-    v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'ref': 'Branch',
-            'fields': {
-                'name': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                'sub_branch': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'default',
-                        'schema': {'type': 'nullable', 'schema': {'type': 'definition-ref', 'schema_ref': 'Branch'}},
-                        'default': None,
-                    },
-                },
-            },
-        }
-    )
-
-    assert v.validate_python({'name': 'root'}) == {'name': 'root', 'sub_branch': None}
-    assert plain_repr(v).startswith(
-        'SchemaValidator(title="typed-dict",validator=DefinitionRef(DefinitionRefValidator{'
-    )
-    assert ',definitions=[TypedDict(TypedDictValidator{' in plain_repr(v)
-
-    assert v.validate_python({'name': 'root', 'sub_branch': {'name': 'b1'}}) == (
-        {'name': 'root', 'sub_branch': {'name': 'b1', 'sub_branch': None}}
-    )
-    assert v.validate_python({'name': 'root', 'sub_branch': {'name': 'b1', 'sub_branch': {'name': 'b2'}}}) == (
-        {'name': 'root', 'sub_branch': {'name': 'b1', 'sub_branch': {'name': 'b2', 'sub_branch': None}}}
-    )
-
-
-def test_branch_nullable_definitions():
     v = SchemaValidator(
         core_schema.definitions_schema(
             {'type': 'definition-ref', 'schema_ref': 'Branch'},
@@ -99,24 +67,25 @@ def test_unused_ref():
 
 def test_nullable_error():
     v = SchemaValidator(
-        {
-            'ref': 'Branch',
-            'type': 'typed-dict',
-            'fields': {
-                'width': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
-                'sub_branch': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'default',
-                        'schema': {
-                            'type': 'union',
-                            'choices': [{'type': 'none'}, {'type': 'definition-ref', 'schema_ref': 'Branch'}],
-                        },
-                        'default': None,
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('Branch'),
+            [
+                core_schema.typed_dict_schema(
+                    {
+                        'width': core_schema.typed_dict_field(core_schema.int_schema()),
+                        'sub_branch': core_schema.typed_dict_field(
+                            core_schema.with_default_schema(
+                                core_schema.union_schema(
+                                    [core_schema.none_schema(), core_schema.definition_reference_schema('Branch')]
+                                ),
+                                default=None,
+                            )
+                        ),
                     },
-                },
-            },
-        }
+                    ref='Branch',
+                )
+            ],
+        )
     )
     assert v.validate_python({'width': 123, 'sub_branch': {'width': 321}}) == (
         {'width': 123, 'sub_branch': {'width': 321, 'sub_branch': None}}
@@ -141,24 +110,23 @@ def test_nullable_error():
 
 def test_list():
     v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'ref': 'BranchList',
-            'fields': {
-                'width': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
-                'branches': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'default',
-                        'schema': {
-                            'type': 'list',
-                            'items_schema': {'type': 'definition-ref', 'schema_ref': 'BranchList'},
-                        },
-                        'default': None,
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('BranchList'),
+            [
+                core_schema.typed_dict_schema(
+                    {
+                        'width': core_schema.typed_dict_field(core_schema.int_schema()),
+                        'branches': core_schema.typed_dict_field(
+                            core_schema.with_default_schema(
+                                core_schema.list_schema(core_schema.definition_reference_schema('BranchList')),
+                                default=None,
+                            )
+                        ),
                     },
-                },
-            },
-        }
+                    ref='BranchList',
+                )
+            ],
+        )
     )
     assert v.validate_python({'width': 1, 'branches': [{'width': 2}, {'width': 3, 'branches': [{'width': 4}]}]}) == (
         {
@@ -183,45 +151,37 @@ def test_multiple_intertwined():
     """
 
     v = SchemaValidator(
-        {
-            'ref': 'Foo',
-            'type': 'typed-dict',
-            'fields': {
-                'height': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
-                'bar': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'ref': 'Bar',
-                        'type': 'typed-dict',
-                        'fields': {
-                            'width': {'type': 'typed-dict-field', 'schema': {'type': 'int'}},
-                            'bars': {
-                                'type': 'typed-dict-field',
-                                'schema': {
-                                    'type': 'default',
-                                    'schema': {
-                                        'type': 'list',
-                                        'items_schema': {'type': 'definition-ref', 'schema_ref': 'Bar'},
-                                    },
-                                    'default': None,
-                                },
-                            },
-                            'foo': {
-                                'type': 'typed-dict-field',
-                                'schema': {
-                                    'type': 'default',
-                                    'schema': {
-                                        'type': 'union',
-                                        'choices': [{'type': 'none'}, {'type': 'definition-ref', 'schema_ref': 'Foo'}],
-                                    },
-                                    'default': None,
-                                },
-                            },
-                        },
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('Foo'),
+            [
+                core_schema.typed_dict_schema(
+                    {
+                        'height': core_schema.typed_dict_field(core_schema.int_schema()),
+                        'bar': core_schema.typed_dict_field(core_schema.definition_reference_schema('Bar')),
                     },
-                },
-            },
-        }
+                    ref='Foo',
+                ),
+                core_schema.typed_dict_schema(
+                    {
+                        'width': core_schema.typed_dict_field(core_schema.int_schema()),
+                        'bars': core_schema.typed_dict_field(
+                            core_schema.with_default_schema(
+                                core_schema.list_schema(core_schema.definition_reference_schema('Bar')), default=None
+                            )
+                        ),
+                        'foo': core_schema.typed_dict_field(
+                            core_schema.with_default_schema(
+                                core_schema.union_schema(
+                                    [core_schema.none_schema(), core_schema.definition_reference_schema('Foo')]
+                                ),
+                                default=None,
+                            )
+                        ),
+                    },
+                    ref='Bar',
+                ),
+            ],
+        )
     )
     v.validate_python(
         {
@@ -244,28 +204,28 @@ def test_model_class():
         branch: Optional['Branch']
 
     v = SchemaValidator(
-        {
-            'type': 'model',
-            'ref': 'Branch',
-            'cls': Branch,
-            'schema': {
-                'type': 'model-fields',
-                'fields': {
-                    'width': {'type': 'model-field', 'schema': {'type': 'int'}},
-                    'branch': {
-                        'type': 'model-field',
-                        'schema': {
-                            'type': 'default',
-                            'schema': {
-                                'type': 'union',
-                                'choices': [{'type': 'none'}, {'type': 'definition-ref', 'schema_ref': 'Branch'}],
-                            },
-                            'default': None,
-                        },
-                    },
-                },
-            },
-        }
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('Branch'),
+            [
+                core_schema.model_schema(
+                    Branch,
+                    core_schema.model_fields_schema(
+                        {
+                            'width': core_schema.model_field(core_schema.int_schema()),
+                            'branch': core_schema.model_field(
+                                core_schema.with_default_schema(
+                                    core_schema.union_schema(
+                                        [core_schema.none_schema(), core_schema.definition_reference_schema('Branch')]
+                                    ),
+                                    default=None,
+                                )
+                            ),
+                        }
+                    ),
+                    ref='Branch',
+                )
+            ],
+        )
     )
     m1: Branch = v.validate_python({'width': '1'})
     assert isinstance(m1, Branch)
@@ -284,7 +244,7 @@ def test_model_class():
 
 
 def test_invalid_schema():
-    with pytest.raises(SchemaError, match='Definitions error: attempted to use `Branch` before it was filled'):
+    with pytest.raises(SchemaError, match='Definitions error: definition `Branch` was never filled'):
         SchemaValidator(
             {
                 'type': 'list',
@@ -311,20 +271,19 @@ def test_invalid_schema():
 
 def test_outside_parent():
     v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'fields': {
-                'tuple1': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'tuple-positional',
-                        'items_schema': [{'type': 'int'}, {'type': 'int'}, {'type': 'str'}],
-                        'ref': 'tuple-iis',
-                    },
-                },
-                'tuple2': {'type': 'typed-dict-field', 'schema': {'type': 'definition-ref', 'schema_ref': 'tuple-iis'}},
-            },
-        }
+        core_schema.definitions_schema(
+            core_schema.typed_dict_schema(
+                {
+                    'tuple1': core_schema.typed_dict_field(core_schema.definition_reference_schema('tuple-iis')),
+                    'tuple2': core_schema.typed_dict_field(core_schema.definition_reference_schema('tuple-iis')),
+                }
+            ),
+            [
+                core_schema.tuple_positional_schema(
+                    [core_schema.int_schema(), core_schema.int_schema(), core_schema.str_schema()], ref='tuple-iis'
+                )
+            ],
+        )
     )
 
     assert v.validate_python({'tuple1': [1, '1', 'frog'], 'tuple2': [2, '2', 'toad']}) == {
@@ -335,21 +294,23 @@ def test_outside_parent():
 
 def test_recursion_branch():
     v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'ref': 'Branch',
-            'fields': {
-                'name': {'type': 'typed-dict-field', 'schema': {'type': 'str'}},
-                'branch': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'default',
-                        'schema': {'type': 'nullable', 'schema': {'type': 'definition-ref', 'schema_ref': 'Branch'}},
-                        'default': None,
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('Branch'),
+            [
+                core_schema.typed_dict_schema(
+                    {
+                        'name': core_schema.typed_dict_field(core_schema.str_schema()),
+                        'branch': core_schema.typed_dict_field(
+                            core_schema.with_default_schema(
+                                core_schema.nullable_schema(core_schema.definition_reference_schema('Branch')),
+                                default=None,
+                            )
+                        ),
                     },
-                },
-            },
-        },
+                    ref='Branch',
+                )
+            ],
+        ),
         {'from_attributes': True},
     )
     assert ',definitions=[TypedDict(TypedDictValidator{' in plain_repr(v)
@@ -377,21 +338,23 @@ def test_recursion_branch():
 
 def test_recursion_branch_from_attributes():
     v = SchemaValidator(
-        {
-            'type': 'model-fields',
-            'ref': 'Branch',
-            'fields': {
-                'name': {'type': 'model-field', 'schema': {'type': 'str'}},
-                'branch': {
-                    'type': 'model-field',
-                    'schema': {
-                        'type': 'default',
-                        'schema': {'type': 'nullable', 'schema': {'type': 'definition-ref', 'schema_ref': 'Branch'}},
-                        'default': None,
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('Branch'),
+            [
+                core_schema.model_fields_schema(
+                    {
+                        'name': core_schema.model_field(core_schema.str_schema()),
+                        'branch': core_schema.model_field(
+                            core_schema.with_default_schema(
+                                core_schema.nullable_schema(core_schema.definition_reference_schema('Branch')),
+                                default=None,
+                            )
+                        ),
                     },
-                },
-            },
-        },
+                    ref='Branch',
+                )
+            ],
+        ),
         {'from_attributes': True},
     )
 
@@ -424,7 +387,10 @@ def test_recursion_branch_from_attributes():
 
 def test_definition_list():
     v = SchemaValidator(
-        {'type': 'list', 'ref': 'the-list', 'items_schema': {'type': 'definition-ref', 'schema_ref': 'the-list'}}
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('the-list'),
+            [core_schema.list_schema(core_schema.definition_reference_schema('the-list'), ref='the-list')],
+        )
     )
     assert ',definitions=[List(ListValidator{' in plain_repr(v)
     assert v.validate_python([]) == []
@@ -448,30 +414,27 @@ def test_definition_list():
 @pytest.fixture(scope='module')
 def multiple_tuple_schema() -> SchemaValidator:
     return SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'fields': {
-                'f1': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'tuple-positional',
-                        'items_schema': [
-                            {'type': 'int'},
-                            {'type': 'nullable', 'schema': {'type': 'definition-ref', 'schema_ref': 't'}},
-                        ],
-                        'ref': 't',
-                    },
-                },
-                'f2': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'default',
-                        'schema': {'type': 'nullable', 'schema': {'type': 'definition-ref', 'schema_ref': 't'}},
-                        'default': None,
-                    },
-                },
-            },
-        }
+        core_schema.definitions_schema(
+            core_schema.typed_dict_schema(
+                {
+                    'f1': core_schema.typed_dict_field(core_schema.definition_reference_schema('t')),
+                    'f2': core_schema.typed_dict_field(
+                        core_schema.with_default_schema(
+                            core_schema.nullable_schema(core_schema.definition_reference_schema('t')), default=None
+                        )
+                    ),
+                }
+            ),
+            [
+                core_schema.tuple_positional_schema(
+                    [
+                        core_schema.int_schema(),
+                        core_schema.nullable_schema(core_schema.definition_reference_schema('t')),
+                    ],
+                    ref='t',
+                )
+            ],
+        )
     )
 
 
@@ -554,18 +517,21 @@ def test_definition_wrap():
         return validator(input_value) + (42,)
 
     v = SchemaValidator(
-        {
-            'type': 'function-wrap',
-            'ref': 'wrapper',
-            'function': {'type': 'general', 'function': wrap_func},
-            'schema': {
-                'type': 'tuple-positional',
-                'items_schema': [
-                    {'type': 'int'},
-                    {'type': 'nullable', 'schema': {'type': 'definition-ref', 'schema_ref': 'wrapper'}},
-                ],
-            },
-        }
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('wrapper'),
+            [
+                core_schema.with_info_wrap_validator_function(
+                    wrap_func,
+                    core_schema.tuple_positional_schema(
+                        [
+                            core_schema.int_schema(),
+                            core_schema.nullable_schema(core_schema.definition_reference_schema('wrapper')),
+                        ]
+                    ),
+                    ref='wrapper',
+                )
+            ],
+        )
     )
     assert v.validate_python((1, None)) == (1, None, 42)
     assert v.validate_python((1, (2, (3, None)))) == (1, (2, (3, None, 42), 42), 42)
@@ -585,19 +551,19 @@ def test_definition_wrap():
 
 def test_union_ref_strictness():
     v = SchemaValidator(
-        {
-            'fields': {
-                'a': {'type': 'typed-dict-field', 'schema': {'type': 'int', 'ref': 'int-type'}},
-                'b': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'union',
-                        'choices': [{'type': 'definition-ref', 'schema_ref': 'int-type'}, {'type': 'str'}],
-                    },
-                },
-            },
-            'type': 'typed-dict',
-        }
+        core_schema.definitions_schema(
+            core_schema.typed_dict_schema(
+                {
+                    'a': core_schema.typed_dict_field(core_schema.definition_reference_schema('int-type')),
+                    'b': core_schema.typed_dict_field(
+                        core_schema.union_schema(
+                            [core_schema.definition_reference_schema('int-type'), core_schema.str_schema()]
+                        )
+                    ),
+                }
+            ),
+            [core_schema.int_schema(ref='int-type')],
+        )
     )
     assert v.validate_python({'a': 1, 'b': '2'}) == {'a': 1, 'b': '2'}
     assert v.validate_python({'a': 1, 'b': 2}) == {'a': 1, 'b': 2}
@@ -613,16 +579,19 @@ def test_union_ref_strictness():
 
 def test_union_container_strictness():
     v = SchemaValidator(
-        {
-            'fields': {
-                'b': {
-                    'type': 'typed-dict-field',
-                    'schema': {'type': 'union', 'choices': [{'type': 'int', 'ref': 'int-type'}, {'type': 'str'}]},
-                },
-                'a': {'type': 'typed-dict-field', 'schema': {'type': 'definition-ref', 'schema_ref': 'int-type'}},
-            },
-            'type': 'typed-dict',
-        }
+        core_schema.definitions_schema(
+            core_schema.typed_dict_schema(
+                {
+                    'b': core_schema.typed_dict_field(
+                        core_schema.union_schema(
+                            [core_schema.definition_reference_schema('int-type'), core_schema.str_schema()]
+                        )
+                    ),
+                    'a': core_schema.typed_dict_field(core_schema.definition_reference_schema('int-type')),
+                }
+            ),
+            [core_schema.int_schema(ref='int-type')],
+        )
     )
     assert v.validate_python({'a': 1, 'b': '2'}) == {'a': 1, 'b': '2'}
     assert v.validate_python({'a': 1, 'b': 2}) == {'a': 1, 'b': 2}
@@ -639,26 +608,25 @@ def test_union_container_strictness():
 @pytest.mark.parametrize('strict', [True, False], ids=lambda s: f'strict={s}')
 def test_union_cycle(strict: bool):
     s = SchemaValidator(
-        {
-            'choices': [
-                {
-                    'fields': {
-                        'foobar': {
-                            'type': 'typed-dict-field',
-                            'schema': {
-                                'items_schema': {'schema_ref': 'root-schema', 'type': 'definition-ref'},
-                                'type': 'list',
-                            },
-                        }
-                    },
-                    'type': 'typed-dict',
-                }
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('root-schema'),
+            [
+                core_schema.union_schema(
+                    [
+                        core_schema.typed_dict_schema(
+                            {
+                                'foobar': core_schema.typed_dict_field(
+                                    core_schema.list_schema(core_schema.definition_reference_schema('root-schema'))
+                                )
+                            }
+                        )
+                    ],
+                    auto_collapse=False,
+                    strict=strict,
+                    ref='root-schema',
+                )
             ],
-            'auto_collapse': False,
-            'strict': strict,
-            'ref': 'root-schema',
-            'type': 'union',
-        }
+        )
     )
 
     data = {'foobar': []}
@@ -681,18 +649,20 @@ def test_function_name():
         return input_value + ' Changed'
 
     v = SchemaValidator(
-        {
-            'choices': [
-                {
-                    'type': 'function-after',
-                    'function': {'type': 'general', 'function': f},
-                    'schema': {'schema_ref': 'root-schema', 'type': 'definition-ref'},
-                },
-                {'type': 'int'},
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('root-schema'),
+            [
+                core_schema.union_schema(
+                    [
+                        core_schema.with_info_after_validator_function(
+                            f, core_schema.definition_reference_schema('root-schema')
+                        ),
+                        core_schema.int_schema(),
+                    ],
+                    ref='root-schema',
+                )
             ],
-            'ref': 'root-schema',
-            'type': 'union',
-        }
+        )
     )
 
     assert v.validate_python(123) == 123
@@ -727,19 +697,21 @@ def test_function_change_id(strict: bool):
         return f'f-{int(count) + 1}'
 
     v = SchemaValidator(
-        {
-            'choices': [
-                {
-                    'type': 'function-before',
-                    'function': {'type': 'general', 'function': f},
-                    'schema': {'schema_ref': 'root-schema', 'type': 'definition-ref'},
-                }
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('root-schema'),
+            [
+                core_schema.union_schema(
+                    [
+                        core_schema.with_info_before_validator_function(
+                            f, core_schema.definition_reference_schema('root-schema')
+                        )
+                    ],
+                    auto_collapse=False,
+                    strict=strict,
+                    ref='root-schema',
+                )
             ],
-            'auto_collapse': False,
-            'strict': strict,
-            'ref': 'root-schema',
-            'type': 'union',
-        }
+        )
     )
 
     with pytest.raises(ValidationError) as exc_info:
@@ -758,23 +730,21 @@ def test_function_change_id(strict: bool):
 def test_many_uses_of_ref():
     # check we can safely exceed RECURSION_GUARD_LIMIT without upsetting the recursion guard
     v = SchemaValidator(
-        {
-            'type': 'typed-dict',
-            'ref': 'Branch',
-            'fields': {
-                'name': {
-                    'type': 'typed-dict-field',
-                    'schema': {'type': 'str', 'max_length': 8, 'ref': 'limited-string'},
-                },
-                'other_names': {
-                    'type': 'typed-dict-field',
-                    'schema': {
-                        'type': 'list',
-                        'items_schema': {'type': 'definition-ref', 'schema_ref': 'limited-string'},
+        core_schema.definitions_schema(
+            core_schema.definition_reference_schema('Branch'),
+            [
+                core_schema.typed_dict_schema(
+                    {
+                        'name': core_schema.typed_dict_field(core_schema.definition_reference_schema('limited-string')),
+                        'other_names': core_schema.typed_dict_field(
+                            core_schema.list_schema(core_schema.definition_reference_schema('limited-string'))
+                        ),
                     },
-                },
-            },
-        }
+                    ref='Branch',
+                ),
+                core_schema.str_schema(max_length=8, ref='limited-string'),
+            ],
+        )
     )
 
     assert v.validate_python({'name': 'Anne', 'other_names': ['Bob', 'Charlie']}) == {
@@ -812,7 +782,8 @@ def test_error_inside_definition_wrapper():
             }
         )
     assert str(exc_info.value) == (
-        'Field "sub_branch":\n'
+        'Error building "typed-dict" validator:\n'
+        '  SchemaError: Field "sub_branch":\n'
         '  SchemaError: Error building "default" validator:\n'
         "  SchemaError: 'default' and 'default_factory' cannot be used together"
     )
@@ -820,7 +791,7 @@ def test_error_inside_definition_wrapper():
 
 def test_recursive_definitions_schema(pydantic_version) -> None:
     s = core_schema.definitions_schema(
-        core_schema.definition_reference_schema(schema_ref='a'),
+        core_schema.definition_reference_schema('a'),
         [
             core_schema.typed_dict_schema(
                 {
@@ -861,7 +832,7 @@ def test_recursive_definitions_schema(pydantic_version) -> None:
 
 def test_unsorted_definitions_schema() -> None:
     s = core_schema.definitions_schema(
-        core_schema.definition_reference_schema(schema_ref='td'),
+        core_schema.definition_reference_schema('td'),
         [
             core_schema.typed_dict_schema(
                 {'x': core_schema.typed_dict_field(core_schema.definition_reference_schema('int'))}, ref='td'
@@ -883,22 +854,28 @@ def test_validate_assignment(pydantic_version) -> None:
     class Model:
         x: List['Model']
 
-    schema = core_schema.dataclass_schema(
-        Model,
-        core_schema.dataclass_args_schema(
-            'Model',
-            [
-                core_schema.dataclass_field(
-                    name='x',
-                    schema=core_schema.list_schema(core_schema.definition_reference_schema('model')),
-                    kw_only=False,
-                )
-            ],
-        ),
-        ['x'],
-        ref='model',
-        config=core_schema.CoreConfig(revalidate_instances='always'),
+    schema = core_schema.definitions_schema(
+        core_schema.definition_reference_schema('model'),
+        [
+            core_schema.dataclass_schema(
+                Model,
+                core_schema.dataclass_args_schema(
+                    'Model',
+                    [
+                        core_schema.dataclass_field(
+                            name='x',
+                            schema=core_schema.list_schema(core_schema.definition_reference_schema('model')),
+                            kw_only=False,
+                        )
+                    ],
+                ),
+                ['x'],
+                ref='model',
+                config=core_schema.CoreConfig(revalidate_instances='always'),
+            )
+        ],
     )
+
     v = SchemaValidator(schema)
 
     data = [Model(x=[Model(x=[])])]
@@ -919,3 +896,211 @@ def test_validate_assignment(pydantic_version) -> None:
             'url': f'https://errors.pydantic.dev/{pydantic_version}/v/dataclass_type',
         }
     ]
+
+
+def test_cyclic_data() -> None:
+    cyclic_data = {}
+    cyclic_data['b'] = {'a': cyclic_data}
+
+    schema = core_schema.definitions_schema(
+        core_schema.definition_reference_schema('a'),
+        [
+            core_schema.typed_dict_schema(
+                {
+                    'b': core_schema.typed_dict_field(
+                        core_schema.nullable_schema(core_schema.definition_reference_schema('b'))
+                    )
+                },
+                ref='a',
+            ),
+            core_schema.typed_dict_schema(
+                {
+                    'a': core_schema.typed_dict_field(
+                        core_schema.nullable_schema(core_schema.definition_reference_schema('a'))
+                    )
+                },
+                ref='b',
+            ),
+        ],
+    )
+
+    validator = SchemaValidator(schema)
+
+    with pytest.raises(ValidationError) as exc_info:
+        validator.validate_python(cyclic_data)
+
+    assert exc_info.value.title == 'typed-dict'
+    assert exc_info.value.errors(include_url=False) == [
+        {
+            'type': 'recursion_loop',
+            'loc': ('b', 'a'),
+            'msg': 'Recursion error - cyclic reference detected',
+            'input': cyclic_data,
+        }
+    ]
+
+
+def test_cyclic_data_threeway() -> None:
+    cyclic_data = {}
+    cyclic_data['b'] = {'c': {'a': cyclic_data}}
+
+    schema = core_schema.definitions_schema(
+        core_schema.definition_reference_schema('a'),
+        [
+            core_schema.typed_dict_schema(
+                {
+                    'b': core_schema.typed_dict_field(
+                        core_schema.nullable_schema(core_schema.definition_reference_schema('b'))
+                    )
+                },
+                ref='a',
+            ),
+            core_schema.typed_dict_schema(
+                {
+                    'c': core_schema.typed_dict_field(
+                        core_schema.nullable_schema(core_schema.definition_reference_schema('c'))
+                    )
+                },
+                ref='b',
+            ),
+            core_schema.typed_dict_schema(
+                {
+                    'a': core_schema.typed_dict_field(
+                        core_schema.nullable_schema(core_schema.definition_reference_schema('a'))
+                    )
+                },
+                ref='c',
+            ),
+        ],
+    )
+
+    validator = SchemaValidator(schema)
+
+    with pytest.raises(ValidationError) as exc_info:
+        validator.validate_python(cyclic_data)
+
+    assert exc_info.value.title == 'typed-dict'
+    assert exc_info.value.errors(include_url=False) == [
+        {
+            'type': 'recursion_loop',
+            'loc': ('b', 'c', 'a'),
+            'msg': 'Recursion error - cyclic reference detected',
+            'input': cyclic_data,
+        }
+    ]
+
+
+def test_complex_recursive_type() -> None:
+    schema = core_schema.definitions_schema(
+        core_schema.definition_reference_schema('JsonType'),
+        [
+            core_schema.nullable_schema(
+                core_schema.union_schema(
+                    [
+                        core_schema.list_schema(core_schema.definition_reference_schema('JsonType')),
+                        core_schema.dict_schema(
+                            core_schema.str_schema(), core_schema.definition_reference_schema('JsonType')
+                        ),
+                        core_schema.str_schema(),
+                        core_schema.int_schema(),
+                        core_schema.float_schema(),
+                        core_schema.bool_schema(),
+                    ]
+                ),
+                ref='JsonType',
+            )
+        ],
+    )
+
+    validator = SchemaValidator(schema)
+
+    with pytest.raises(ValidationError) as exc_info:
+        validator.validate_python({'a': datetime.date(year=1992, month=12, day=11)})
+
+    assert exc_info.value.errors(include_url=False) == [
+        {
+            'type': 'list_type',
+            'loc': ('list[nullable[union[list[...],dict[str,...],str,int,float,bool]]]',),
+            'msg': 'Input should be a valid list',
+            'input': {'a': datetime.date(1992, 12, 11)},
+        },
+        {
+            'type': 'list_type',
+            'loc': ('dict[str,...]', 'a', 'list[nullable[union[list[...],dict[str,...],str,int,float,bool]]]'),
+            'msg': 'Input should be a valid list',
+            'input': datetime.date(1992, 12, 11),
+        },
+        {
+            'type': 'dict_type',
+            'loc': ('dict[str,...]', 'a', 'dict[str,...]'),
+            'msg': 'Input should be a valid dictionary',
+            'input': datetime.date(1992, 12, 11),
+        },
+        {
+            'type': 'string_type',
+            'loc': ('dict[str,...]', 'a', 'str'),
+            'msg': 'Input should be a valid string',
+            'input': datetime.date(1992, 12, 11),
+        },
+        {
+            'type': 'int_type',
+            'loc': ('dict[str,...]', 'a', 'int'),
+            'msg': 'Input should be a valid integer',
+            'input': datetime.date(1992, 12, 11),
+        },
+        {
+            'type': 'float_type',
+            'loc': ('dict[str,...]', 'a', 'float'),
+            'msg': 'Input should be a valid number',
+            'input': datetime.date(1992, 12, 11),
+        },
+        {
+            'type': 'bool_type',
+            'loc': ('dict[str,...]', 'a', 'bool'),
+            'msg': 'Input should be a valid boolean',
+            'input': datetime.date(1992, 12, 11),
+        },
+        {
+            'type': 'string_type',
+            'loc': ('str',),
+            'msg': 'Input should be a valid string',
+            'input': {'a': datetime.date(1992, 12, 11)},
+        },
+        {
+            'type': 'int_type',
+            'loc': ('int',),
+            'msg': 'Input should be a valid integer',
+            'input': {'a': datetime.date(1992, 12, 11)},
+        },
+        {
+            'type': 'float_type',
+            'loc': ('float',),
+            'msg': 'Input should be a valid number',
+            'input': {'a': datetime.date(1992, 12, 11)},
+        },
+        {
+            'type': 'bool_type',
+            'loc': ('bool',),
+            'msg': 'Input should be a valid boolean',
+            'input': {'a': datetime.date(1992, 12, 11)},
+        },
+    ]
+
+
+def test_no_exponential_blowup():
+    """See https://github.com/pydantic/pydantic/issues/8049
+
+    There was a performance bug which led to exponential blowup when trying to
+    build a schema with many intermingled recursive definitions.
+    """
+    unions = core_schema.union_schema([core_schema.definition_reference_schema(f'foo_{i}') for i in range(100)])
+
+    schema = core_schema.definitions_schema(
+        core_schema.typed_dict_schema({'x': core_schema.typed_dict_field(unions)}),
+        definitions=[
+            core_schema.typed_dict_schema({'a': core_schema.typed_dict_field(unions)}, ref=f'foo_{i}')
+            for i in range(100)
+        ],
+    )
+
+    SchemaValidator(schema)
