@@ -6,7 +6,7 @@ from typing import Any, Dict
 import pytest
 from dirty_equals import IsStr
 
-from pydantic_core import SchemaValidator, ValidationError
+from pydantic_core import SchemaValidator, ValidationError, core_schema
 
 from ..conftest import Err, PyAndJson, plain_repr
 
@@ -51,14 +51,16 @@ def test_int_py_and_json(py_and_json: PyAndJson, input_value, expected):
     else:
         output = v.validate_test(input_value)
         assert output == expected
-        assert isinstance(output, int)
+        assert type(output) == int
 
 
 @pytest.mark.parametrize(
     'input_value,expected',
     [
         (Decimal('1'), 1),
+        (Decimal('1' + '0' * 1_000), int('1' + '0' * 1_000)),  # a large decimal
         (Decimal('1.0'), 1),
+        (1.0, 1),
         (i64_max, i64_max),
         (str(i64_max), i64_max),
         (str(i64_max * 2), i64_max * 2),
@@ -66,6 +68,14 @@ def test_int_py_and_json(py_and_json: PyAndJson, input_value, expected):
         (-i64_max + 1, -i64_max + 1),
         (i64_max * 2, i64_max * 2),
         (-i64_max * 2, -i64_max * 2),
+        pytest.param(
+            1.00000000001,
+            Err(
+                'Input should be a valid integer, got a number with a fractional part '
+                '[type=int_from_float, input_value=1.00000000001, input_type=float]'
+            ),
+            id='decimal-remainder',
+        ),
         pytest.param(
             Decimal('1.001'),
             Err(
@@ -347,10 +357,10 @@ def test_too_long(pydantic_version):
     ]
     # insert_assert(repr(exc_info.value))
     assert repr(exc_info.value) == (
-        "1 validation error for int\n"
-        "  Unable to parse input string as an integer, exceeded maximum size "
+        '1 validation error for int\n'
+        '  Unable to parse input string as an integer, exceeded maximum size '
         "[type=int_parsing_size, input_value='111111111111111111111111...11111111111111111111111', input_type=str]\n"
-        f"    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/int_parsing_size"
+        f'    For further information visit https://errors.pydantic.dev/{pydantic_version}/v/int_parsing_size'
     )
 
 
@@ -392,7 +402,8 @@ def test_int_key(py_and_json: PyAndJson):
     v = py_and_json({'type': 'dict', 'keys_schema': {'type': 'int'}, 'values_schema': {'type': 'int'}})
     assert v.validate_test({'1': 1, '2': 2}) == {1: 1, 2: 2}
     with pytest.raises(ValidationError, match='Input should be a valid integer'):
-        v.validate_test({'1': 1, '2': 2}, strict=True)
+        v.validate_python({'1': 1, '2': 2}, strict=True)
+    assert v.validate_json('{"1": 1, "2": 2}', strict=True) == {1: 1, 2: 2}
 
 
 def test_string_as_int_with_underscores() -> None:
@@ -405,3 +416,83 @@ def test_string_as_int_with_underscores() -> None:
             v.validate_python(edge_case)
         with pytest.raises(ValidationError):
             v.validate_json(f'"{edge_case}"')
+
+
+class IntSubclass(int):
+    pass
+
+
+def test_int_subclass() -> None:
+    v = SchemaValidator({'type': 'int'})
+    v_lax = v.validate_python(IntSubclass(1))
+    assert v_lax == 1
+    assert type(v_lax) == int
+    v_strict = v.validate_python(IntSubclass(1), strict=True)
+    assert v_strict == 1
+    assert type(v_strict) == int
+
+    assert v.validate_python(IntSubclass(1136885225876639845)) == 1136885225876639845
+    assert v.validate_python(IntSubclass(i64_max + 7)) == i64_max + 7
+    assert v.validate_python(IntSubclass(1136885225876639845), strict=True) == 1136885225876639845
+    assert v.validate_python(IntSubclass(i64_max + 7), strict=True) == i64_max + 7
+
+
+def test_int_subclass_constraint() -> None:
+    v = SchemaValidator({'type': 'int', 'gt': 0})
+    v_lax = v.validate_python(IntSubclass(1))
+    assert v_lax == 1
+    assert type(v_lax) == int
+    v_strict = v.validate_python(IntSubclass(1), strict=True)
+    assert v_strict == 1
+    assert type(v_strict) == int
+
+    with pytest.raises(ValidationError, match='Input should be greater than 0'):
+        v.validate_python(IntSubclass(0))
+
+
+class FloatSubclass(float):
+    pass
+
+
+def test_float_subclass() -> None:
+    v = SchemaValidator({'type': 'int'})
+    v_lax = v.validate_python(FloatSubclass(1))
+    assert v_lax == 1
+    assert type(v_lax) == int
+
+
+def test_int_subclass_plain_enum() -> None:
+    v = SchemaValidator({'type': 'int'})
+
+    from enum import Enum
+
+    class PlainEnum(Enum):
+        ONE = 1
+
+    v_lax = v.validate_python(PlainEnum.ONE)
+    assert v_lax == 1
+    assert type(v_lax) == int
+
+
+def test_allow_inf_nan_true_json() -> None:
+    v = SchemaValidator(core_schema.int_schema(), core_schema.CoreConfig(allow_inf_nan=True))
+
+    assert v.validate_json('123') == 123
+    with pytest.raises(ValidationError, match=r'Input should be a finite number \[type=finite_number'):
+        v.validate_json('NaN')
+    with pytest.raises(ValidationError, match=r'Input should be a finite number \[type=finite_number'):
+        v.validate_json('Infinity')
+    with pytest.raises(ValidationError, match=r'Input should be a finite number \[type=finite_number'):
+        v.validate_json('-Infinity')
+
+
+def test_allow_inf_nan_false_json() -> None:
+    v = SchemaValidator(core_schema.int_schema(), core_schema.CoreConfig(allow_inf_nan=False))
+
+    assert v.validate_json('123') == 123
+    with pytest.raises(ValidationError, match=r'Input should be a finite number \[type=finite_number'):
+        v.validate_json('NaN')
+    with pytest.raises(ValidationError, match=r'Input should be a finite number \[type=finite_number'):
+        v.validate_json('Infinity')
+    with pytest.raises(ValidationError, match=r'Input should be a finite number \[type=finite_number'):
+        v.validate_json('-Infinity')
